/****************************************************************************
** Meta object code from reading C++ file 'translation.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../appstream/qt/translation.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'translation.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9AppStream11TranslationE_t {};
} // unnamed namespace

template <> constexpr inline auto AppStream::Translation::qt_create_metaobjectdata<qt_meta_tag_ZN9AppStream11TranslationE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AppStream::Translation",
        "Kind",
        "KindUnknown",
        "KindGettext",
        "KindQt"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Kind'
        QtMocHelpers::EnumData<enum Kind>(1, 1, QMC::EnumFlags{}).add({
            {    2, Kind::KindUnknown },
            {    3, Kind::KindGettext },
            {    4, Kind::KindQt },
        }),
    };
    return QtMocHelpers::metaObjectData<Translation, qt_meta_tag_ZN9AppStream11TranslationE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AppStream::Translation::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9AppStream11TranslationE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9AppStream11TranslationE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9AppStream11TranslationE_t>.metaTypes,
    nullptr
} };

QT_WARNING_POP
