
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "as-enum-types.h"
#include "appstream.h"
#include "as-agreement.h"
#include "as-agreement-section.h"
#include "as-artifact.h"
#include "as-branding.h"
#include "as-bundle.h"
#include "as-category.h"
#include "as-checksum.h"
#include "as-component.h"
#include "as-component-box.h"
#include "as-content-rating.h"
#include "as-context.h"
#include "as-developer.h"
#include "as-icon.h"
#include "as-image.h"
#include "as-issue.h"
#include "as-launchable.h"
#include "as-macros.h"
#include "as-metadata.h"
#include "as-pool.h"
#include "as-provided.h"
#include "as-reference.h"
#include "as-relation.h"
#include "as-relation-check-result.h"
#include "as-release.h"
#include "as-release-list.h"
#include "as-review.h"
#include "as-reviews-client.h"
#include "as-screenshot.h"
#include "as-spdx.h"
#include "as-suggested.h"
#include "as-system-info.h"
#include "as-translation.h"
#include "as-utils.h"
#include "as-validator.h"
#include "as-validator-issue.h"
#include "as-vercmp.h"
#include "as-video.h"

#define C_ENUM(v) ((gint) v)
#define C_FLAGS(v) ((guint) v)

/* enumerations from "as-agreement.h" */

GType
as_agreement_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_AGREEMENT_KIND_UNKNOWN), "AS_AGREEMENT_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_AGREEMENT_KIND_GENERIC), "AS_AGREEMENT_KIND_GENERIC", "generic" },
        { C_ENUM (AS_AGREEMENT_KIND_EULA), "AS_AGREEMENT_KIND_EULA", "eula" },
        { C_ENUM (AS_AGREEMENT_KIND_PRIVACY), "AS_AGREEMENT_KIND_PRIVACY", "privacy" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsAgreementKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "as-artifact.h" */

GType
as_size_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_SIZE_KIND_UNKNOWN), "AS_SIZE_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_SIZE_KIND_DOWNLOAD), "AS_SIZE_KIND_DOWNLOAD", "download" },
        { C_ENUM (AS_SIZE_KIND_INSTALLED), "AS_SIZE_KIND_INSTALLED", "installed" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsSizeKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
as_artifact_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_ARTIFACT_KIND_UNKNOWN), "AS_ARTIFACT_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_ARTIFACT_KIND_SOURCE), "AS_ARTIFACT_KIND_SOURCE", "source" },
        { C_ENUM (AS_ARTIFACT_KIND_BINARY), "AS_ARTIFACT_KIND_BINARY", "binary" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsArtifactKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "as-branding.h" */

GType
as_color_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_COLOR_KIND_UNKNOWN), "AS_COLOR_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_COLOR_KIND_PRIMARY), "AS_COLOR_KIND_PRIMARY", "primary" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsColorKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
as_color_scheme_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_COLOR_SCHEME_KIND_UNKNOWN), "AS_COLOR_SCHEME_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_COLOR_SCHEME_KIND_LIGHT), "AS_COLOR_SCHEME_KIND_LIGHT", "light" },
        { C_ENUM (AS_COLOR_SCHEME_KIND_DARK), "AS_COLOR_SCHEME_KIND_DARK", "dark" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsColorSchemeKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "as-bundle.h" */

GType
as_bundle_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_BUNDLE_KIND_UNKNOWN), "AS_BUNDLE_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_BUNDLE_KIND_PACKAGE), "AS_BUNDLE_KIND_PACKAGE", "package" },
        { C_ENUM (AS_BUNDLE_KIND_LIMBA), "AS_BUNDLE_KIND_LIMBA", "limba" },
        { C_ENUM (AS_BUNDLE_KIND_FLATPAK), "AS_BUNDLE_KIND_FLATPAK", "flatpak" },
        { C_ENUM (AS_BUNDLE_KIND_APPIMAGE), "AS_BUNDLE_KIND_APPIMAGE", "appimage" },
        { C_ENUM (AS_BUNDLE_KIND_SNAP), "AS_BUNDLE_KIND_SNAP", "snap" },
        { C_ENUM (AS_BUNDLE_KIND_TARBALL), "AS_BUNDLE_KIND_TARBALL", "tarball" },
        { C_ENUM (AS_BUNDLE_KIND_CABINET), "AS_BUNDLE_KIND_CABINET", "cabinet" },
        { C_ENUM (AS_BUNDLE_KIND_LINGLONG), "AS_BUNDLE_KIND_LINGLONG", "linglong" },
        { C_ENUM (AS_BUNDLE_KIND_SYSUPDATE), "AS_BUNDLE_KIND_SYSUPDATE", "sysupdate" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsBundleKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "as-checksum.h" */

GType
as_checksum_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_CHECKSUM_KIND_NONE), "AS_CHECKSUM_KIND_NONE", "none" },
        { C_ENUM (AS_CHECKSUM_KIND_SHA1), "AS_CHECKSUM_KIND_SHA1", "sha1" },
        { C_ENUM (AS_CHECKSUM_KIND_SHA256), "AS_CHECKSUM_KIND_SHA256", "sha256" },
        { C_ENUM (AS_CHECKSUM_KIND_SHA512), "AS_CHECKSUM_KIND_SHA512", "sha512" },
        { C_ENUM (AS_CHECKSUM_KIND_BLAKE2B), "AS_CHECKSUM_KIND_BLAKE2B", "blake2b" },
        { C_ENUM (AS_CHECKSUM_KIND_BLAKE3), "AS_CHECKSUM_KIND_BLAKE3", "blake3" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsChecksumKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "as-component-box.h" */

GType
as_component_box_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (AS_COMPONENT_BOX_FLAG_NONE), "AS_COMPONENT_BOX_FLAG_NONE", "none" },
        { C_FLAGS (AS_COMPONENT_BOX_FLAG_NO_CHECKS), "AS_COMPONENT_BOX_FLAG_NO_CHECKS", "no-checks" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("AsComponentBoxFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "as-component.h" */

GType
as_component_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_COMPONENT_KIND_UNKNOWN), "AS_COMPONENT_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_COMPONENT_KIND_GENERIC), "AS_COMPONENT_KIND_GENERIC", "generic" },
        { C_ENUM (AS_COMPONENT_KIND_DESKTOP_APP), "AS_COMPONENT_KIND_DESKTOP_APP", "desktop-app" },
        { C_ENUM (AS_COMPONENT_KIND_CONSOLE_APP), "AS_COMPONENT_KIND_CONSOLE_APP", "console-app" },
        { C_ENUM (AS_COMPONENT_KIND_WEB_APP), "AS_COMPONENT_KIND_WEB_APP", "web-app" },
        { C_ENUM (AS_COMPONENT_KIND_SERVICE), "AS_COMPONENT_KIND_SERVICE", "service" },
        { C_ENUM (AS_COMPONENT_KIND_ADDON), "AS_COMPONENT_KIND_ADDON", "addon" },
        { C_ENUM (AS_COMPONENT_KIND_RUNTIME), "AS_COMPONENT_KIND_RUNTIME", "runtime" },
        { C_ENUM (AS_COMPONENT_KIND_FONT), "AS_COMPONENT_KIND_FONT", "font" },
        { C_ENUM (AS_COMPONENT_KIND_CODEC), "AS_COMPONENT_KIND_CODEC", "codec" },
        { C_ENUM (AS_COMPONENT_KIND_INPUT_METHOD), "AS_COMPONENT_KIND_INPUT_METHOD", "input-method" },
        { C_ENUM (AS_COMPONENT_KIND_OPERATING_SYSTEM), "AS_COMPONENT_KIND_OPERATING_SYSTEM", "operating-system" },
        { C_ENUM (AS_COMPONENT_KIND_FIRMWARE), "AS_COMPONENT_KIND_FIRMWARE", "firmware" },
        { C_ENUM (AS_COMPONENT_KIND_DRIVER), "AS_COMPONENT_KIND_DRIVER", "driver" },
        { C_ENUM (AS_COMPONENT_KIND_LOCALIZATION), "AS_COMPONENT_KIND_LOCALIZATION", "localization" },
        { C_ENUM (AS_COMPONENT_KIND_REPOSITORY), "AS_COMPONENT_KIND_REPOSITORY", "repository" },
        { C_ENUM (AS_COMPONENT_KIND_ICON_THEME), "AS_COMPONENT_KIND_ICON_THEME", "icon-theme" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsComponentKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
as_merge_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_MERGE_KIND_NONE), "AS_MERGE_KIND_NONE", "none" },
        { C_ENUM (AS_MERGE_KIND_REPLACE), "AS_MERGE_KIND_REPLACE", "replace" },
        { C_ENUM (AS_MERGE_KIND_APPEND), "AS_MERGE_KIND_APPEND", "append" },
        { C_ENUM (AS_MERGE_KIND_REMOVE_COMPONENT), "AS_MERGE_KIND_REMOVE_COMPONENT", "remove-component" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsMergeKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
as_component_scope_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_COMPONENT_SCOPE_UNKNOWN), "AS_COMPONENT_SCOPE_UNKNOWN", "unknown" },
        { C_ENUM (AS_COMPONENT_SCOPE_SYSTEM), "AS_COMPONENT_SCOPE_SYSTEM", "system" },
        { C_ENUM (AS_COMPONENT_SCOPE_USER), "AS_COMPONENT_SCOPE_USER", "user" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsComponentScope"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
as_url_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_URL_KIND_UNKNOWN), "AS_URL_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_URL_KIND_HOMEPAGE), "AS_URL_KIND_HOMEPAGE", "homepage" },
        { C_ENUM (AS_URL_KIND_BUGTRACKER), "AS_URL_KIND_BUGTRACKER", "bugtracker" },
        { C_ENUM (AS_URL_KIND_FAQ), "AS_URL_KIND_FAQ", "faq" },
        { C_ENUM (AS_URL_KIND_HELP), "AS_URL_KIND_HELP", "help" },
        { C_ENUM (AS_URL_KIND_DONATION), "AS_URL_KIND_DONATION", "donation" },
        { C_ENUM (AS_URL_KIND_TRANSLATE), "AS_URL_KIND_TRANSLATE", "translate" },
        { C_ENUM (AS_URL_KIND_CONTACT), "AS_URL_KIND_CONTACT", "contact" },
        { C_ENUM (AS_URL_KIND_VCS_BROWSER), "AS_URL_KIND_VCS_BROWSER", "vcs-browser" },
        { C_ENUM (AS_URL_KIND_CONTRIBUTE), "AS_URL_KIND_CONTRIBUTE", "contribute" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsUrlKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "as-content-rating.h" */

GType
as_content_rating_system_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_CONTENT_RATING_SYSTEM_UNKNOWN), "AS_CONTENT_RATING_SYSTEM_UNKNOWN", "unknown" },
        { C_ENUM (AS_CONTENT_RATING_SYSTEM_INCAA), "AS_CONTENT_RATING_SYSTEM_INCAA", "incaa" },
        { C_ENUM (AS_CONTENT_RATING_SYSTEM_ACB), "AS_CONTENT_RATING_SYSTEM_ACB", "acb" },
        { C_ENUM (AS_CONTENT_RATING_SYSTEM_DJCTQ), "AS_CONTENT_RATING_SYSTEM_DJCTQ", "djctq" },
        { C_ENUM (AS_CONTENT_RATING_SYSTEM_GSRR), "AS_CONTENT_RATING_SYSTEM_GSRR", "gsrr" },
        { C_ENUM (AS_CONTENT_RATING_SYSTEM_PEGI), "AS_CONTENT_RATING_SYSTEM_PEGI", "pegi" },
        { C_ENUM (AS_CONTENT_RATING_SYSTEM_KAVI), "AS_CONTENT_RATING_SYSTEM_KAVI", "kavi" },
        { C_ENUM (AS_CONTENT_RATING_SYSTEM_USK), "AS_CONTENT_RATING_SYSTEM_USK", "usk" },
        { C_ENUM (AS_CONTENT_RATING_SYSTEM_ESRA), "AS_CONTENT_RATING_SYSTEM_ESRA", "esra" },
        { C_ENUM (AS_CONTENT_RATING_SYSTEM_CERO), "AS_CONTENT_RATING_SYSTEM_CERO", "cero" },
        { C_ENUM (AS_CONTENT_RATING_SYSTEM_OFLCNZ), "AS_CONTENT_RATING_SYSTEM_OFLCNZ", "oflcnz" },
        { C_ENUM (AS_CONTENT_RATING_SYSTEM_RUSSIA), "AS_CONTENT_RATING_SYSTEM_RUSSIA", "russia" },
        { C_ENUM (AS_CONTENT_RATING_SYSTEM_MDA), "AS_CONTENT_RATING_SYSTEM_MDA", "mda" },
        { C_ENUM (AS_CONTENT_RATING_SYSTEM_GRAC), "AS_CONTENT_RATING_SYSTEM_GRAC", "grac" },
        { C_ENUM (AS_CONTENT_RATING_SYSTEM_ESRB), "AS_CONTENT_RATING_SYSTEM_ESRB", "esrb" },
        { C_ENUM (AS_CONTENT_RATING_SYSTEM_IARC), "AS_CONTENT_RATING_SYSTEM_IARC", "iarc" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsContentRatingSystem"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
as_content_rating_value_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_CONTENT_RATING_VALUE_UNKNOWN), "AS_CONTENT_RATING_VALUE_UNKNOWN", "unknown" },
        { C_ENUM (AS_CONTENT_RATING_VALUE_NONE), "AS_CONTENT_RATING_VALUE_NONE", "none" },
        { C_ENUM (AS_CONTENT_RATING_VALUE_MILD), "AS_CONTENT_RATING_VALUE_MILD", "mild" },
        { C_ENUM (AS_CONTENT_RATING_VALUE_MODERATE), "AS_CONTENT_RATING_VALUE_MODERATE", "moderate" },
        { C_ENUM (AS_CONTENT_RATING_VALUE_INTENSE), "AS_CONTENT_RATING_VALUE_INTENSE", "intense" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsContentRatingValue"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "as-context.h" */

GType
as_format_version_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_FORMAT_VERSION_UNKNOWN), "AS_FORMAT_VERSION_UNKNOWN", "unknown" },
        { C_ENUM (AS_FORMAT_VERSION_V1_0), "AS_FORMAT_VERSION_V1_0", "v1-0" },
        { C_ENUM (AS_FORMAT_VERSION_V1_2), "AS_FORMAT_VERSION_V1_2", "v1-2" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsFormatVersion"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
as_format_style_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_FORMAT_STYLE_UNKNOWN), "AS_FORMAT_STYLE_UNKNOWN", "unknown" },
        { C_ENUM (AS_FORMAT_STYLE_METAINFO), "AS_FORMAT_STYLE_METAINFO", "metainfo" },
        { C_ENUM (AS_FORMAT_STYLE_CATALOG), "AS_FORMAT_STYLE_CATALOG", "catalog" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsFormatStyle"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
as_format_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_FORMAT_KIND_UNKNOWN), "AS_FORMAT_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_FORMAT_KIND_XML), "AS_FORMAT_KIND_XML", "xml" },
        { C_ENUM (AS_FORMAT_KIND_YAML), "AS_FORMAT_KIND_YAML", "yaml" },
        { C_ENUM (AS_FORMAT_KIND_DESKTOP_ENTRY), "AS_FORMAT_KIND_DESKTOP_ENTRY", "desktop-entry" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsFormatKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
as_value_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (AS_VALUE_FLAG_NONE), "AS_VALUE_FLAG_NONE", "none" },
        { C_FLAGS (AS_VALUE_FLAG_DUPLICATE_CHECK), "AS_VALUE_FLAG_DUPLICATE_CHECK", "duplicate-check" },
        { C_FLAGS (AS_VALUE_FLAG_NO_TRANSLATION_FALLBACK), "AS_VALUE_FLAG_NO_TRANSLATION_FALLBACK", "no-translation-fallback" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("AsValueFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "as-icon.h" */

GType
as_icon_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_ICON_KIND_UNKNOWN), "AS_ICON_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_ICON_KIND_STOCK), "AS_ICON_KIND_STOCK", "stock" },
        { C_ENUM (AS_ICON_KIND_CACHED), "AS_ICON_KIND_CACHED", "cached" },
        { C_ENUM (AS_ICON_KIND_LOCAL), "AS_ICON_KIND_LOCAL", "local" },
        { C_ENUM (AS_ICON_KIND_REMOTE), "AS_ICON_KIND_REMOTE", "remote" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsIconKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "as-image.h" */

GType
as_image_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_IMAGE_KIND_UNKNOWN), "AS_IMAGE_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_IMAGE_KIND_SOURCE), "AS_IMAGE_KIND_SOURCE", "source" },
        { C_ENUM (AS_IMAGE_KIND_THUMBNAIL), "AS_IMAGE_KIND_THUMBNAIL", "thumbnail" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsImageKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "as-issue.h" */

GType
as_issue_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_ISSUE_KIND_UNKNOWN), "AS_ISSUE_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_ISSUE_KIND_GENERIC), "AS_ISSUE_KIND_GENERIC", "generic" },
        { C_ENUM (AS_ISSUE_KIND_CVE), "AS_ISSUE_KIND_CVE", "cve" },
        { C_ENUM (AS_ISSUE_KIND_GCVE), "AS_ISSUE_KIND_GCVE", "gcve" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsIssueKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "as-launchable.h" */

GType
as_launchable_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_LAUNCHABLE_KIND_UNKNOWN), "AS_LAUNCHABLE_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_LAUNCHABLE_KIND_DESKTOP_ID), "AS_LAUNCHABLE_KIND_DESKTOP_ID", "desktop-id" },
        { C_ENUM (AS_LAUNCHABLE_KIND_SERVICE), "AS_LAUNCHABLE_KIND_SERVICE", "service" },
        { C_ENUM (AS_LAUNCHABLE_KIND_COCKPIT_MANIFEST), "AS_LAUNCHABLE_KIND_COCKPIT_MANIFEST", "cockpit-manifest" },
        { C_ENUM (AS_LAUNCHABLE_KIND_URL), "AS_LAUNCHABLE_KIND_URL", "url" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsLaunchableKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "as-metadata.h" */

GType
as_parse_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (AS_PARSE_FLAG_NONE), "AS_PARSE_FLAG_NONE", "none" },
        { C_FLAGS (AS_PARSE_FLAG_IGNORE_MEDIABASEURL), "AS_PARSE_FLAG_IGNORE_MEDIABASEURL", "ignore-mediabaseurl" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("AsParseFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
as_metadata_error_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_METADATA_ERROR_FAILED), "AS_METADATA_ERROR_FAILED", "failed" },
        { C_ENUM (AS_METADATA_ERROR_PARSE), "AS_METADATA_ERROR_PARSE", "parse" },
        { C_ENUM (AS_METADATA_ERROR_FORMAT_UNEXPECTED), "AS_METADATA_ERROR_FORMAT_UNEXPECTED", "format-unexpected" },
        { C_ENUM (AS_METADATA_ERROR_NO_COMPONENT), "AS_METADATA_ERROR_NO_COMPONENT", "no-component" },
        { C_ENUM (AS_METADATA_ERROR_VALUE_MISSING), "AS_METADATA_ERROR_VALUE_MISSING", "value-missing" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsMetadataError"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "as-pool.h" */

GType
as_pool_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (AS_POOL_FLAG_NONE), "AS_POOL_FLAG_NONE", "none" },
        { C_FLAGS (AS_POOL_FLAG_LOAD_OS_CATALOG), "AS_POOL_FLAG_LOAD_OS_CATALOG", "load-os-catalog" },
        { C_FLAGS (AS_POOL_FLAG_LOAD_OS_METAINFO), "AS_POOL_FLAG_LOAD_OS_METAINFO", "load-os-metainfo" },
        { C_FLAGS (AS_POOL_FLAG_LOAD_OS_DESKTOP_FILES), "AS_POOL_FLAG_LOAD_OS_DESKTOP_FILES", "load-os-desktop-files" },
        { C_FLAGS (AS_POOL_FLAG_LOAD_FLATPAK), "AS_POOL_FLAG_LOAD_FLATPAK", "load-flatpak" },
        { C_FLAGS (AS_POOL_FLAG_IGNORE_CACHE_AGE), "AS_POOL_FLAG_IGNORE_CACHE_AGE", "ignore-cache-age" },
        { C_FLAGS (AS_POOL_FLAG_RESOLVE_ADDONS), "AS_POOL_FLAG_RESOLVE_ADDONS", "resolve-addons" },
        { C_FLAGS (AS_POOL_FLAG_PREFER_OS_METAINFO), "AS_POOL_FLAG_PREFER_OS_METAINFO", "prefer-os-metainfo" },
        { C_FLAGS (AS_POOL_FLAG_MONITOR), "AS_POOL_FLAG_MONITOR", "monitor" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("AsPoolFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
as_cache_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (AS_CACHE_FLAG_NONE), "AS_CACHE_FLAG_NONE", "none" },
        { C_FLAGS (AS_CACHE_FLAG_USE_USER), "AS_CACHE_FLAG_USE_USER", "use-user" },
        { C_FLAGS (AS_CACHE_FLAG_USE_SYSTEM), "AS_CACHE_FLAG_USE_SYSTEM", "use-system" },
        { C_FLAGS (AS_CACHE_FLAG_NO_CLEAR), "AS_CACHE_FLAG_NO_CLEAR", "no-clear" },
        { C_FLAGS (AS_CACHE_FLAG_REFRESH_SYSTEM), "AS_CACHE_FLAG_REFRESH_SYSTEM", "refresh-system" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("AsCacheFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
as_pool_error_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_POOL_ERROR_FAILED), "AS_POOL_ERROR_FAILED", "failed" },
        { C_ENUM (AS_POOL_ERROR_INCOMPLETE), "AS_POOL_ERROR_INCOMPLETE", "incomplete" },
        { C_ENUM (AS_POOL_ERROR_COLLISION), "AS_POOL_ERROR_COLLISION", "collision" },
        { C_ENUM (AS_POOL_ERROR_CACHE_WRITE_FAILED), "AS_POOL_ERROR_CACHE_WRITE_FAILED", "cache-write-failed" },
        { C_ENUM (AS_POOL_ERROR_CACHE_DAMAGED), "AS_POOL_ERROR_CACHE_DAMAGED", "cache-damaged" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsPoolError"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "as-provided.h" */

GType
as_provided_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_PROVIDED_KIND_UNKNOWN), "AS_PROVIDED_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_PROVIDED_KIND_LIBRARY), "AS_PROVIDED_KIND_LIBRARY", "library" },
        { C_ENUM (AS_PROVIDED_KIND_BINARY), "AS_PROVIDED_KIND_BINARY", "binary" },
        { C_ENUM (AS_PROVIDED_KIND_MEDIATYPE), "AS_PROVIDED_KIND_MEDIATYPE", "mediatype" },
        { C_ENUM (AS_PROVIDED_KIND_FONT), "AS_PROVIDED_KIND_FONT", "font" },
        { C_ENUM (AS_PROVIDED_KIND_MODALIAS), "AS_PROVIDED_KIND_MODALIAS", "modalias" },
        { C_ENUM (AS_PROVIDED_KIND_PYTHON), "AS_PROVIDED_KIND_PYTHON", "python" },
        { C_ENUM (AS_PROVIDED_KIND_DBUS_SYSTEM), "AS_PROVIDED_KIND_DBUS_SYSTEM", "dbus-system" },
        { C_ENUM (AS_PROVIDED_KIND_DBUS_USER), "AS_PROVIDED_KIND_DBUS_USER", "dbus-user" },
        { C_ENUM (AS_PROVIDED_KIND_FIRMWARE_RUNTIME), "AS_PROVIDED_KIND_FIRMWARE_RUNTIME", "firmware-runtime" },
        { C_ENUM (AS_PROVIDED_KIND_FIRMWARE_FLASHED), "AS_PROVIDED_KIND_FIRMWARE_FLASHED", "firmware-flashed" },
        { C_ENUM (AS_PROVIDED_KIND_ID), "AS_PROVIDED_KIND_ID", "id" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsProvidedKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "as-reference.h" */

GType
as_reference_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_REFERENCE_KIND_UNKNOWN), "AS_REFERENCE_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_REFERENCE_KIND_DOI), "AS_REFERENCE_KIND_DOI", "doi" },
        { C_ENUM (AS_REFERENCE_KIND_CITATION_CFF), "AS_REFERENCE_KIND_CITATION_CFF", "citation-cff" },
        { C_ENUM (AS_REFERENCE_KIND_REGISTRY), "AS_REFERENCE_KIND_REGISTRY", "registry" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsReferenceKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "as-relation-check-result.h" */

GType
as_relation_status_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_RELATION_STATUS_UNKNOWN), "AS_RELATION_STATUS_UNKNOWN", "unknown" },
        { C_ENUM (AS_RELATION_STATUS_ERROR), "AS_RELATION_STATUS_ERROR", "error" },
        { C_ENUM (AS_RELATION_STATUS_NOT_SATISFIED), "AS_RELATION_STATUS_NOT_SATISFIED", "not-satisfied" },
        { C_ENUM (AS_RELATION_STATUS_SATISFIED), "AS_RELATION_STATUS_SATISFIED", "satisfied" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsRelationStatus"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "as-relation.h" */

GType
as_relation_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_RELATION_KIND_UNKNOWN), "AS_RELATION_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_RELATION_KIND_REQUIRES), "AS_RELATION_KIND_REQUIRES", "requires" },
        { C_ENUM (AS_RELATION_KIND_RECOMMENDS), "AS_RELATION_KIND_RECOMMENDS", "recommends" },
        { C_ENUM (AS_RELATION_KIND_SUPPORTS), "AS_RELATION_KIND_SUPPORTS", "supports" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsRelationKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
as_relation_item_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_RELATION_ITEM_KIND_UNKNOWN), "AS_RELATION_ITEM_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_RELATION_ITEM_KIND_ID), "AS_RELATION_ITEM_KIND_ID", "id" },
        { C_ENUM (AS_RELATION_ITEM_KIND_MODALIAS), "AS_RELATION_ITEM_KIND_MODALIAS", "modalias" },
        { C_ENUM (AS_RELATION_ITEM_KIND_KERNEL), "AS_RELATION_ITEM_KIND_KERNEL", "kernel" },
        { C_ENUM (AS_RELATION_ITEM_KIND_MEMORY), "AS_RELATION_ITEM_KIND_MEMORY", "memory" },
        { C_ENUM (AS_RELATION_ITEM_KIND_FIRMWARE), "AS_RELATION_ITEM_KIND_FIRMWARE", "firmware" },
        { C_ENUM (AS_RELATION_ITEM_KIND_CONTROL), "AS_RELATION_ITEM_KIND_CONTROL", "control" },
        { C_ENUM (AS_RELATION_ITEM_KIND_DISPLAY_LENGTH), "AS_RELATION_ITEM_KIND_DISPLAY_LENGTH", "display-length" },
        { C_ENUM (AS_RELATION_ITEM_KIND_HARDWARE), "AS_RELATION_ITEM_KIND_HARDWARE", "hardware" },
        { C_ENUM (AS_RELATION_ITEM_KIND_INTERNET), "AS_RELATION_ITEM_KIND_INTERNET", "internet" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsRelationItemKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
as_relation_compare_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_RELATION_COMPARE_UNKNOWN), "AS_RELATION_COMPARE_UNKNOWN", "unknown" },
        { C_ENUM (AS_RELATION_COMPARE_EQ), "AS_RELATION_COMPARE_EQ", "eq" },
        { C_ENUM (AS_RELATION_COMPARE_NE), "AS_RELATION_COMPARE_NE", "ne" },
        { C_ENUM (AS_RELATION_COMPARE_LT), "AS_RELATION_COMPARE_LT", "lt" },
        { C_ENUM (AS_RELATION_COMPARE_GT), "AS_RELATION_COMPARE_GT", "gt" },
        { C_ENUM (AS_RELATION_COMPARE_LE), "AS_RELATION_COMPARE_LE", "le" },
        { C_ENUM (AS_RELATION_COMPARE_GE), "AS_RELATION_COMPARE_GE", "ge" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsRelationCompare"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
as_control_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_CONTROL_KIND_UNKNOWN), "AS_CONTROL_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_CONTROL_KIND_POINTING), "AS_CONTROL_KIND_POINTING", "pointing" },
        { C_ENUM (AS_CONTROL_KIND_KEYBOARD), "AS_CONTROL_KIND_KEYBOARD", "keyboard" },
        { C_ENUM (AS_CONTROL_KIND_CONSOLE), "AS_CONTROL_KIND_CONSOLE", "console" },
        { C_ENUM (AS_CONTROL_KIND_TOUCH), "AS_CONTROL_KIND_TOUCH", "touch" },
        { C_ENUM (AS_CONTROL_KIND_GAMEPAD), "AS_CONTROL_KIND_GAMEPAD", "gamepad" },
        { C_ENUM (AS_CONTROL_KIND_VOICE), "AS_CONTROL_KIND_VOICE", "voice" },
        { C_ENUM (AS_CONTROL_KIND_VISION), "AS_CONTROL_KIND_VISION", "vision" },
        { C_ENUM (AS_CONTROL_KIND_TV_REMOTE), "AS_CONTROL_KIND_TV_REMOTE", "tv-remote" },
        { C_ENUM (AS_CONTROL_KIND_TABLET), "AS_CONTROL_KIND_TABLET", "tablet" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsControlKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
as_display_side_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_DISPLAY_SIDE_KIND_UNKNOWN), "AS_DISPLAY_SIDE_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_DISPLAY_SIDE_KIND_SHORTEST), "AS_DISPLAY_SIDE_KIND_SHORTEST", "shortest" },
        { C_ENUM (AS_DISPLAY_SIDE_KIND_LONGEST), "AS_DISPLAY_SIDE_KIND_LONGEST", "longest" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsDisplaySideKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
as_internet_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_INTERNET_KIND_UNKNOWN), "AS_INTERNET_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_INTERNET_KIND_ALWAYS), "AS_INTERNET_KIND_ALWAYS", "always" },
        { C_ENUM (AS_INTERNET_KIND_OFFLINE_ONLY), "AS_INTERNET_KIND_OFFLINE_ONLY", "offline-only" },
        { C_ENUM (AS_INTERNET_KIND_FIRST_RUN), "AS_INTERNET_KIND_FIRST_RUN", "first-run" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsInternetKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
as_relation_error_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_RELATION_ERROR_FAILED), "AS_RELATION_ERROR_FAILED", "failed" },
        { C_ENUM (AS_RELATION_ERROR_BAD_VALUE), "AS_RELATION_ERROR_BAD_VALUE", "bad-value" },
        { C_ENUM (AS_RELATION_ERROR_NOT_IMPLEMENTED), "AS_RELATION_ERROR_NOT_IMPLEMENTED", "not-implemented" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsRelationError"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "as-release-list.h" */

GType
as_release_list_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_RELEASE_LIST_KIND_UNKNOWN), "AS_RELEASE_LIST_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_RELEASE_LIST_KIND_EMBEDDED), "AS_RELEASE_LIST_KIND_EMBEDDED", "embedded" },
        { C_ENUM (AS_RELEASE_LIST_KIND_EXTERNAL), "AS_RELEASE_LIST_KIND_EXTERNAL", "external" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsReleaseListKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "as-release.h" */

GType
as_release_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_RELEASE_KIND_UNKNOWN), "AS_RELEASE_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_RELEASE_KIND_STABLE), "AS_RELEASE_KIND_STABLE", "stable" },
        { C_ENUM (AS_RELEASE_KIND_DEVELOPMENT), "AS_RELEASE_KIND_DEVELOPMENT", "development" },
        { C_ENUM (AS_RELEASE_KIND_SNAPSHOT), "AS_RELEASE_KIND_SNAPSHOT", "snapshot" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsReleaseKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
as_urgency_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_URGENCY_KIND_UNKNOWN), "AS_URGENCY_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_URGENCY_KIND_LOW), "AS_URGENCY_KIND_LOW", "low" },
        { C_ENUM (AS_URGENCY_KIND_MEDIUM), "AS_URGENCY_KIND_MEDIUM", "medium" },
        { C_ENUM (AS_URGENCY_KIND_HIGH), "AS_URGENCY_KIND_HIGH", "high" },
        { C_ENUM (AS_URGENCY_KIND_CRITICAL), "AS_URGENCY_KIND_CRITICAL", "critical" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsUrgencyKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
as_release_url_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_RELEASE_URL_KIND_UNKNOWN), "AS_RELEASE_URL_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_RELEASE_URL_KIND_DETAILS), "AS_RELEASE_URL_KIND_DETAILS", "details" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsReleaseUrlKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "as-review.h" */

GType
as_review_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (AS_REVIEW_FLAG_NONE), "AS_REVIEW_FLAG_NONE", "none" },
        { C_FLAGS (AS_REVIEW_FLAG_SELF), "AS_REVIEW_FLAG_SELF", "self" },
        { C_FLAGS (AS_REVIEW_FLAG_VOTED), "AS_REVIEW_FLAG_VOTED", "voted" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("AsReviewFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "as-reviews-client.h" */

GType
as_reviews_client_error_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_REVIEWS_CLIENT_ERROR_FAILED), "AS_REVIEWS_CLIENT_ERROR_FAILED", "failed" },
        { C_ENUM (AS_REVIEWS_CLIENT_ERROR_NETWORK), "AS_REVIEWS_CLIENT_ERROR_NETWORK", "network" },
        { C_ENUM (AS_REVIEWS_CLIENT_ERROR_PARSE), "AS_REVIEWS_CLIENT_ERROR_PARSE", "parse" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsReviewsClientError"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
as_review_vote_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_REVIEW_VOTE_KIND_UNKNOWN), "AS_REVIEW_VOTE_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_REVIEW_VOTE_KIND_UP), "AS_REVIEW_VOTE_KIND_UP", "up" },
        { C_ENUM (AS_REVIEW_VOTE_KIND_DOWN), "AS_REVIEW_VOTE_KIND_DOWN", "down" },
        { C_ENUM (AS_REVIEW_VOTE_KIND_REPORT), "AS_REVIEW_VOTE_KIND_REPORT", "report" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsReviewVoteKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "as-screenshot.h" */

GType
as_screenshot_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_SCREENSHOT_KIND_UNKNOWN), "AS_SCREENSHOT_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_SCREENSHOT_KIND_DEFAULT), "AS_SCREENSHOT_KIND_DEFAULT", "default" },
        { C_ENUM (AS_SCREENSHOT_KIND_EXTRA), "AS_SCREENSHOT_KIND_EXTRA", "extra" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsScreenshotKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
as_screenshot_media_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_SCREENSHOT_MEDIA_KIND_UNKNOWN), "AS_SCREENSHOT_MEDIA_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_SCREENSHOT_MEDIA_KIND_IMAGE), "AS_SCREENSHOT_MEDIA_KIND_IMAGE", "image" },
        { C_ENUM (AS_SCREENSHOT_MEDIA_KIND_VIDEO), "AS_SCREENSHOT_MEDIA_KIND_VIDEO", "video" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsScreenshotMediaKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "as-suggested.h" */

GType
as_suggested_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_SUGGESTED_KIND_UNKNOWN), "AS_SUGGESTED_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_SUGGESTED_KIND_UPSTREAM), "AS_SUGGESTED_KIND_UPSTREAM", "upstream" },
        { C_ENUM (AS_SUGGESTED_KIND_HEURISTIC), "AS_SUGGESTED_KIND_HEURISTIC", "heuristic" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsSuggestedKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "as-system-info.h" */

GType
as_check_result_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_CHECK_RESULT_ERROR), "AS_CHECK_RESULT_ERROR", "error" },
        { C_ENUM (AS_CHECK_RESULT_UNKNOWN), "AS_CHECK_RESULT_UNKNOWN", "unknown" },
        { C_ENUM (AS_CHECK_RESULT_FALSE), "AS_CHECK_RESULT_FALSE", "false" },
        { C_ENUM (AS_CHECK_RESULT_TRUE), "AS_CHECK_RESULT_TRUE", "true" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsCheckResult"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
as_chassis_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_CHASSIS_KIND_UNKNOWN), "AS_CHASSIS_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_CHASSIS_KIND_DESKTOP), "AS_CHASSIS_KIND_DESKTOP", "desktop" },
        { C_ENUM (AS_CHASSIS_KIND_LAPTOP), "AS_CHASSIS_KIND_LAPTOP", "laptop" },
        { C_ENUM (AS_CHASSIS_KIND_SERVER), "AS_CHASSIS_KIND_SERVER", "server" },
        { C_ENUM (AS_CHASSIS_KIND_TABLET), "AS_CHASSIS_KIND_TABLET", "tablet" },
        { C_ENUM (AS_CHASSIS_KIND_HANDSET), "AS_CHASSIS_KIND_HANDSET", "handset" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsChassisKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
as_system_info_error_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_SYSTEM_INFO_ERROR_FAILED), "AS_SYSTEM_INFO_ERROR_FAILED", "failed" },
        { C_ENUM (AS_SYSTEM_INFO_ERROR_NOT_FOUND), "AS_SYSTEM_INFO_ERROR_NOT_FOUND", "not-found" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsSystemInfoError"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "as-translation.h" */

GType
as_translation_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_TRANSLATION_KIND_UNKNOWN), "AS_TRANSLATION_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_TRANSLATION_KIND_GETTEXT), "AS_TRANSLATION_KIND_GETTEXT", "gettext" },
        { C_ENUM (AS_TRANSLATION_KIND_QT), "AS_TRANSLATION_KIND_QT", "qt" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsTranslationKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "as-utils.h" */

GType
as_utils_error_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_UTILS_ERROR_FAILED), "AS_UTILS_ERROR_FAILED", "failed" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsUtilsError"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
as_data_id_match_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (AS_DATA_ID_MATCH_FLAG_NONE), "AS_DATA_ID_MATCH_FLAG_NONE", "none" },
        { C_FLAGS (AS_DATA_ID_MATCH_FLAG_SCOPE), "AS_DATA_ID_MATCH_FLAG_SCOPE", "scope" },
        { C_FLAGS (AS_DATA_ID_MATCH_FLAG_BUNDLE_KIND), "AS_DATA_ID_MATCH_FLAG_BUNDLE_KIND", "bundle-kind" },
        { C_FLAGS (AS_DATA_ID_MATCH_FLAG_ORIGIN), "AS_DATA_ID_MATCH_FLAG_ORIGIN", "origin" },
        { C_FLAGS (AS_DATA_ID_MATCH_FLAG_ID), "AS_DATA_ID_MATCH_FLAG_ID", "id" },
        { C_FLAGS (AS_DATA_ID_MATCH_FLAG_BRANCH), "AS_DATA_ID_MATCH_FLAG_BRANCH", "branch" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("AsDataIdMatchFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
as_metadata_location_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_METADATA_LOCATION_UNKNOWN), "AS_METADATA_LOCATION_UNKNOWN", "unknown" },
        { C_ENUM (AS_METADATA_LOCATION_SHARED), "AS_METADATA_LOCATION_SHARED", "shared" },
        { C_ENUM (AS_METADATA_LOCATION_STATE), "AS_METADATA_LOCATION_STATE", "state" },
        { C_ENUM (AS_METADATA_LOCATION_CACHE), "AS_METADATA_LOCATION_CACHE", "cache" },
        { C_ENUM (AS_METADATA_LOCATION_USER), "AS_METADATA_LOCATION_USER", "user" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsMetadataLocation"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
as_markup_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_MARKUP_KIND_UNKNOWN), "AS_MARKUP_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_MARKUP_KIND_XML), "AS_MARKUP_KIND_XML", "xml" },
        { C_ENUM (AS_MARKUP_KIND_TEXT), "AS_MARKUP_KIND_TEXT", "text" },
        { C_ENUM (AS_MARKUP_KIND_MARKDOWN), "AS_MARKUP_KIND_MARKDOWN", "markdown" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsMarkupKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "as-validator-issue.h" */

GType
as_issue_severity_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_ISSUE_SEVERITY_UNKNOWN), "AS_ISSUE_SEVERITY_UNKNOWN", "unknown" },
        { C_ENUM (AS_ISSUE_SEVERITY_PEDANTIC), "AS_ISSUE_SEVERITY_PEDANTIC", "pedantic" },
        { C_ENUM (AS_ISSUE_SEVERITY_INFO), "AS_ISSUE_SEVERITY_INFO", "info" },
        { C_ENUM (AS_ISSUE_SEVERITY_WARNING), "AS_ISSUE_SEVERITY_WARNING", "warning" },
        { C_ENUM (AS_ISSUE_SEVERITY_ERROR), "AS_ISSUE_SEVERITY_ERROR", "error" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsIssueSeverity"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "as-validator.h" */

GType
as_validator_error_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_VALIDATOR_ERROR_FAILED), "AS_VALIDATOR_ERROR_FAILED", "failed" },
        { C_ENUM (AS_VALIDATOR_ERROR_INVALID_OVERRIDE), "AS_VALIDATOR_ERROR_INVALID_OVERRIDE", "invalid-override" },
        { C_ENUM (AS_VALIDATOR_ERROR_INVALID_FILENAME), "AS_VALIDATOR_ERROR_INVALID_FILENAME", "invalid-filename" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsValidatorError"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "as-vercmp.h" */

GType
as_vercmp_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (AS_VERCMP_FLAG_NONE), "AS_VERCMP_FLAG_NONE", "none" },
        { C_FLAGS (AS_VERCMP_FLAG_IGNORE_EPOCH), "AS_VERCMP_FLAG_IGNORE_EPOCH", "ignore-epoch" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("AsVercmpFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "as-video.h" */

GType
as_video_codec_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_VIDEO_CODEC_KIND_UNKNOWN), "AS_VIDEO_CODEC_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_VIDEO_CODEC_KIND_VP9), "AS_VIDEO_CODEC_KIND_VP9", "vp9" },
        { C_ENUM (AS_VIDEO_CODEC_KIND_AV1), "AS_VIDEO_CODEC_KIND_AV1", "av1" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsVideoCodecKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
as_video_container_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (AS_VIDEO_CONTAINER_KIND_UNKNOWN), "AS_VIDEO_CONTAINER_KIND_UNKNOWN", "unknown" },
        { C_ENUM (AS_VIDEO_CONTAINER_KIND_MKV), "AS_VIDEO_CONTAINER_KIND_MKV", "mkv" },
        { C_ENUM (AS_VIDEO_CONTAINER_KIND_WEBM), "AS_VIDEO_CONTAINER_KIND_WEBM", "webm" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AsVideoContainerKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* Generated data ends here */

