
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "asc-enums-types.h"
#include "appstream-compose.h"
#include "asc-compose.h"
#include "asc-directory-unit.h"
#include "asc-globals.h"
#include "asc-hint.h"
#include "asc-icon-policy.h"
#include "asc-media.h"
#include "asc-result.h"
#include "asc-unit.h"
#include "asc-utils.h"

#define C_ENUM(v) ((gint) v)
#define C_FLAGS(v) ((guint) v)

/* enumerations from "asc-compose.h" */

GType
asc_compose_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (ASC_COMPOSE_FLAG_NONE), "ASC_COMPOSE_FLAG_NONE", "none" },
        { C_FLAGS (ASC_COMPOSE_FLAG_USE_THREADS), "ASC_COMPOSE_FLAG_USE_THREADS", "use-threads" },
        { C_FLAGS (ASC_COMPOSE_FLAG_ALLOW_NET), "ASC_COMPOSE_FLAG_ALLOW_NET", "allow-net" },
        { C_FLAGS (ASC_COMPOSE_FLAG_VALIDATE), "ASC_COMPOSE_FLAG_VALIDATE", "validate" },
        { C_FLAGS (ASC_COMPOSE_FLAG_STORE_SCREENSHOTS), "ASC_COMPOSE_FLAG_STORE_SCREENSHOTS", "store-screenshots" },
        { C_FLAGS (ASC_COMPOSE_FLAG_ALLOW_VIDEOS), "ASC_COMPOSE_FLAG_ALLOW_VIDEOS", "allow-videos" },
        { C_FLAGS (ASC_COMPOSE_FLAG_PROCESS_FONTS), "ASC_COMPOSE_FLAG_PROCESS_FONTS", "process-fonts" },
        { C_FLAGS (ASC_COMPOSE_FLAG_PROCESS_TRANSLATIONS), "ASC_COMPOSE_FLAG_PROCESS_TRANSLATIONS", "process-translations" },
        { C_FLAGS (ASC_COMPOSE_FLAG_IGNORE_ICONS), "ASC_COMPOSE_FLAG_IGNORE_ICONS", "ignore-icons" },
        { C_FLAGS (ASC_COMPOSE_FLAG_PROCESS_UNPAIRED_DESKTOP), "ASC_COMPOSE_FLAG_PROCESS_UNPAIRED_DESKTOP", "process-unpaired-desktop" },
        { C_FLAGS (ASC_COMPOSE_FLAG_PROPAGATE_CUSTOM), "ASC_COMPOSE_FLAG_PROPAGATE_CUSTOM", "propagate-custom" },
        { C_FLAGS (ASC_COMPOSE_FLAG_PROPAGATE_ARTIFACTS), "ASC_COMPOSE_FLAG_PROPAGATE_ARTIFACTS", "propagate-artifacts" },
        { C_FLAGS (ASC_COMPOSE_FLAG_NO_FINAL_CHECK), "ASC_COMPOSE_FLAG_NO_FINAL_CHECK", "no-final-check" },
        { C_FLAGS (ASC_COMPOSE_FLAG_NO_PARTIAL_URLS), "ASC_COMPOSE_FLAG_NO_PARTIAL_URLS", "no-partial-urls" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("AscComposeFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "asc-globals.h" */

GType
asc_compose_error_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (ASC_COMPOSE_ERROR_FAILED), "ASC_COMPOSE_ERROR_FAILED", "failed" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AscComposeError"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "asc-icon-policy.h" */

GType
asc_icon_state_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (ASC_ICON_STATE_IGNORED), "ASC_ICON_STATE_IGNORED", "ignored" },
        { C_ENUM (ASC_ICON_STATE_CACHED_REMOTE), "ASC_ICON_STATE_CACHED_REMOTE", "cached-remote" },
        { C_ENUM (ASC_ICON_STATE_CACHED_ONLY), "ASC_ICON_STATE_CACHED_ONLY", "cached-only" },
        { C_ENUM (ASC_ICON_STATE_REMOTE_ONLY), "ASC_ICON_STATE_REMOTE_ONLY", "remote-only" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AscIconState"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "asc-media.h" */

GType
asc_media_error_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (ASC_MEDIA_ERROR_FAILED), "ASC_MEDIA_ERROR_FAILED", "failed" },
        { C_ENUM (ASC_MEDIA_ERROR_DEAD_WORKER), "ASC_MEDIA_ERROR_DEAD_WORKER", "dead-worker" },
        { C_ENUM (ASC_MEDIA_ERROR_TIMEOUT), "ASC_MEDIA_ERROR_TIMEOUT", "timeout" },
        { C_ENUM (ASC_MEDIA_ERROR_PROTOCOL), "ASC_MEDIA_ERROR_PROTOCOL", "protocol" },
        { C_ENUM (ASC_MEDIA_ERROR_UNSUPPORTED), "ASC_MEDIA_ERROR_UNSUPPORTED", "unsupported" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AscMediaError"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
asc_image_scale_mode_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (ASC_IMAGE_SCALE_MODE_NONE), "ASC_IMAGE_SCALE_MODE_NONE", "none" },
        { C_ENUM (ASC_IMAGE_SCALE_MODE_PAD), "ASC_IMAGE_SCALE_MODE_PAD", "pad" },
        { C_ENUM (ASC_IMAGE_SCALE_MODE_FIT_WIDTH), "ASC_IMAGE_SCALE_MODE_FIT_WIDTH", "fit-width" },
        { C_ENUM (ASC_IMAGE_SCALE_MODE_FIT_HEIGHT), "ASC_IMAGE_SCALE_MODE_FIT_HEIGHT", "fit-height" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AscImageScaleMode"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
asc_image_save_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (ASC_IMAGE_SAVE_FLAG_NONE), "ASC_IMAGE_SAVE_FLAG_NONE", "none" },
        { C_FLAGS (ASC_IMAGE_SAVE_FLAG_LOSSLESS), "ASC_IMAGE_SAVE_FLAG_LOSSLESS", "lossless" },
        { C_FLAGS (ASC_IMAGE_SAVE_FLAG_OPTIMIZE), "ASC_IMAGE_SAVE_FLAG_OPTIMIZE", "optimize" },
        { C_FLAGS (ASC_IMAGE_SAVE_FLAG_SHARPEN), "ASC_IMAGE_SAVE_FLAG_SHARPEN", "sharpen" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("AscImageSaveFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
asc_image_format_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (ASC_IMAGE_FORMAT_UNKNOWN), "ASC_IMAGE_FORMAT_UNKNOWN", "unknown" },
        { C_ENUM (ASC_IMAGE_FORMAT_PNG), "ASC_IMAGE_FORMAT_PNG", "png" },
        { C_ENUM (ASC_IMAGE_FORMAT_JXL), "ASC_IMAGE_FORMAT_JXL", "jxl" },
        { C_ENUM (ASC_IMAGE_FORMAT_SVG), "ASC_IMAGE_FORMAT_SVG", "svg" },
        { C_ENUM (ASC_IMAGE_FORMAT_SVGZ), "ASC_IMAGE_FORMAT_SVGZ", "svgz" },
        { C_ENUM (ASC_IMAGE_FORMAT_AVIF), "ASC_IMAGE_FORMAT_AVIF", "avif" },
        { C_ENUM (ASC_IMAGE_FORMAT_WEBP), "ASC_IMAGE_FORMAT_WEBP", "webp" },
        { C_ENUM (ASC_IMAGE_FORMAT_JPEG), "ASC_IMAGE_FORMAT_JPEG", "jpeg" },
        { C_ENUM (ASC_IMAGE_FORMAT_GIF), "ASC_IMAGE_FORMAT_GIF", "gif" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("AscImageFormat"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* Generated data ends here */

