/****************************************************************************
** Meta object code from reading C++ file 'relation.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../appstream/qt/relation.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'relation.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9AppStream8RelationE_t {};
} // unnamed namespace

template <> constexpr inline auto AppStream::Relation::qt_create_metaobjectdata<qt_meta_tag_ZN9AppStream8RelationE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AppStream::Relation",
        "Kind",
        "KindUnknown",
        "KindRequires",
        "KindRecommends",
        "KindSupports",
        "ItemKind",
        "ItemKindUnknown",
        "ItemKindId",
        "ItemKindModalias",
        "ItemKindKernel",
        "ItemKindMemory",
        "ItemKindFirmware",
        "ItemKindControl",
        "ItemKindDisplayLength",
        "ItemKindHardware",
        "ItemKindInternet",
        "Compare",
        "CompareUnknown",
        "CompareEq",
        "CompareNe",
        "CompareLt",
        "CompareGt",
        "CompareLe",
        "CompareGe",
        "ControlKind",
        "ControlKindUnknown",
        "ControlKindPointing",
        "ControlKindKeyboard",
        "ControlKindConsole",
        "ControlKindTouch",
        "ControlKindGamepad",
        "ControlKindVoice",
        "ControlKindVision",
        "ControlKindTvRemote",
        "ControlKindTablet",
        "DisplaySideKind",
        "DisplaySideKindUnknown",
        "DisplaySideKindShortest",
        "DisplaySideKindLongest",
        "InternetKind",
        "InternetKindUnknown",
        "InternetKindAlways",
        "InternetKindOfflineOnly",
        "InternetKindFirstRun"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Kind'
        QtMocHelpers::EnumData<enum Kind>(1, 1, QMC::EnumFlags{}).add({
            {    2, Kind::KindUnknown },
            {    3, Kind::KindRequires },
            {    4, Kind::KindRecommends },
            {    5, Kind::KindSupports },
        }),
        // enum 'ItemKind'
        QtMocHelpers::EnumData<enum ItemKind>(6, 6, QMC::EnumFlags{}).add({
            {    7, ItemKind::ItemKindUnknown },
            {    8, ItemKind::ItemKindId },
            {    9, ItemKind::ItemKindModalias },
            {   10, ItemKind::ItemKindKernel },
            {   11, ItemKind::ItemKindMemory },
            {   12, ItemKind::ItemKindFirmware },
            {   13, ItemKind::ItemKindControl },
            {   14, ItemKind::ItemKindDisplayLength },
            {   15, ItemKind::ItemKindHardware },
            {   16, ItemKind::ItemKindInternet },
        }),
        // enum 'Compare'
        QtMocHelpers::EnumData<enum Compare>(17, 17, QMC::EnumFlags{}).add({
            {   18, Compare::CompareUnknown },
            {   19, Compare::CompareEq },
            {   20, Compare::CompareNe },
            {   21, Compare::CompareLt },
            {   22, Compare::CompareGt },
            {   23, Compare::CompareLe },
            {   24, Compare::CompareGe },
        }),
        // enum 'ControlKind'
        QtMocHelpers::EnumData<enum ControlKind>(25, 25, QMC::EnumFlags{}).add({
            {   26, ControlKind::ControlKindUnknown },
            {   27, ControlKind::ControlKindPointing },
            {   28, ControlKind::ControlKindKeyboard },
            {   29, ControlKind::ControlKindConsole },
            {   30, ControlKind::ControlKindTouch },
            {   31, ControlKind::ControlKindGamepad },
            {   32, ControlKind::ControlKindVoice },
            {   33, ControlKind::ControlKindVision },
            {   34, ControlKind::ControlKindTvRemote },
            {   35, ControlKind::ControlKindTablet },
        }),
        // enum 'DisplaySideKind'
        QtMocHelpers::EnumData<enum DisplaySideKind>(36, 36, QMC::EnumFlags{}).add({
            {   37, DisplaySideKind::DisplaySideKindUnknown },
            {   38, DisplaySideKind::DisplaySideKindShortest },
            {   39, DisplaySideKind::DisplaySideKindLongest },
        }),
        // enum 'InternetKind'
        QtMocHelpers::EnumData<enum InternetKind>(40, 40, QMC::EnumFlags{}).add({
            {   41, InternetKind::InternetKindUnknown },
            {   42, InternetKind::InternetKindAlways },
            {   43, InternetKind::InternetKindOfflineOnly },
            {   44, InternetKind::InternetKindFirstRun },
        }),
    };
    return QtMocHelpers::metaObjectData<Relation, qt_meta_tag_ZN9AppStream8RelationE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AppStream::Relation::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9AppStream8RelationE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9AppStream8RelationE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9AppStream8RelationE_t>.metaTypes,
    nullptr
} };

QT_WARNING_POP
