/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
 *
 * This file was automatically generated.
 */

#pragma once

/* DO NOT EDIT THIS FILE MANUALLY! */

/*
 * Desktop environment names from: data/desktop-environments.txt
 * Environment/Style names from:   data/desktop-style-ids.txt
 */

#include <glib/gi18n-lib.h>
#include "as-macros-private.h"

AS_BEGIN_PRIVATE_DECLS
/* clang-format off */

/**
 * AsDesktopEnvData:
 *
 * Registered desktop environments.
 */
typedef struct {
	const gchar	*id;
	const gchar	*name;
} AsDesktopEnvData;

/**
 * AsGUIEnvStyleData:
 *
 * Graphical environment/style IDs.
 */
typedef struct {
	const gchar	*id;
	const gchar	*name;
} AsGUIEnvStyleData;

AsDesktopEnvData as_desktop_env_data[] = {
	/* TRANSLATORS: Name of the "Cinnamon" desktop environment. */
	{ "Cinnamon", N_("Cinnamon") },
	/* TRANSLATORS: Name of the "DDE" desktop environment. */
	{ "DDE", N_("Deepin") },
	/* TRANSLATORS: Name of the "EDE" desktop environment. */
	{ "EDE", N_("EDE") },
	/* TRANSLATORS: Name of the "Endless" desktop environment. */
	{ "Endless", N_("Endless") },
	/* TRANSLATORS: Name of the "GNOME" desktop environment. */
	{ "GNOME", N_("GNOME") },
	/* TRANSLATORS: Name of the "KDE" desktop environment. */
	{ "KDE", N_("KDE Plasma") },
	/* TRANSLATORS: Name of the "LXDE" desktop environment. */
	{ "LXDE", N_("LXDE") },
	/* TRANSLATORS: Name of the "LXQt" desktop environment. */
	{ "LXQt", N_("LXQt") },
	/* TRANSLATORS: Name of the "MATE" desktop environment. */
	{ "MATE", N_("MATE") },
	/* TRANSLATORS: Name of the "Pantheon" desktop environment. */
	{ "Pantheon", N_("Pantheon") },
	/* TRANSLATORS: Name of the "Plasma" desktop environment. */
	{ "Plasma", N_("KDE Plasma") },
	/* TRANSLATORS: Name of the "Razor" desktop environment. */
	{ "Razor", N_("Razor") },
	/* TRANSLATORS: Name of the "ROX" desktop environment. */
	{ "ROX", N_("ROX") },
	/* TRANSLATORS: Name of the "Unity" desktop environment. */
	{ "Unity", N_("Unity") },
	/* TRANSLATORS: Name of the "XFCE" desktop environment. */
	{ "XFCE", N_("Xfce") },
	{ NULL, NULL },
};

AsGUIEnvStyleData as_gui_env_style_data[] = {
	/* TRANSLATORS: Name of the "android" visual environment style. */
	{ "android", N_("Android") },
	/* TRANSLATORS: Name of the "cinnamon" visual environment style. */
	{ "cinnamon", N_("Cinnamon") },
	/* TRANSLATORS: Name of the "dde" visual environment style. */
	{ "dde", N_("Deepin") },
	/* TRANSLATORS: Name of the "ede" visual environment style. */
	{ "ede", N_("EDE") },
	/* TRANSLATORS: Name of the "gnome" visual environment style. */
	{ "gnome", N_("GNOME") },
	/* TRANSLATORS: Name of the "gnome:dark" visual environment style. */
	{ "gnome:dark", N_("GNOME (Dark)") },
	/* TRANSLATORS: Name of the "ios" visual environment style. */
	{ "ios", N_("iOS") },
	/* TRANSLATORS: Name of the "lxde" visual environment style. */
	{ "lxde", N_("LXDE") },
	/* TRANSLATORS: Name of the "lxqt" visual environment style. */
	{ "lxqt", N_("LXQt") },
	/* TRANSLATORS: Name of the "macos" visual environment style. */
	{ "macos", N_("macOS") },
	/* TRANSLATORS: Name of the "mate" visual environment style. */
	{ "mate", N_("Mate") },
	/* TRANSLATORS: Name of the "pantheon" visual environment style. */
	{ "pantheon", N_("Pantheon") },
	/* TRANSLATORS: Name of the "pantheon:dark" visual environment style. */
	{ "pantheon:dark", N_("Pantheon (Dark)") },
	/* TRANSLATORS: Name of the "plasma" visual environment style. */
	{ "plasma", N_("KDE Plasma") },
	/* TRANSLATORS: Name of the "plasma-mobile" visual environment style. */
	{ "plasma-mobile", N_("Plasma Mobile") },
	/* TRANSLATORS: Name of the "plasma-mobile:dark" visual environment style. */
	{ "plasma-mobile:dark", N_("Plasma Mobile (Dark)") },
	/* TRANSLATORS: Name of the "plasma:dark" visual environment style. */
	{ "plasma:dark", N_("KDE Plasma (Dark)") },
	/* TRANSLATORS: Name of the "razor" visual environment style. */
	{ "razor", N_("Razor") },
	/* TRANSLATORS: Name of the "rox" visual environment style. */
	{ "rox", N_("Rox") },
	/* TRANSLATORS: Name of the "unity" visual environment style. */
	{ "unity", N_("Unity") },
	/* TRANSLATORS: Name of the "windows" visual environment style. */
	{ "windows", N_("Microsoft Windows") },
	/* TRANSLATORS: Name of the "xfce" visual environment style. */
	{ "xfce", N_("Xfce") },
	{ NULL, NULL },
};

/* clang-format on */
AS_END_PRIVATE_DECLS
