/****************************************************************************
** Meta object code from reading C++ file 'metadata.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../appstream/qt/metadata.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'metadata.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9AppStream8MetadataE_t {};
} // unnamed namespace

template <> constexpr inline auto AppStream::Metadata::qt_create_metaobjectdata<qt_meta_tag_ZN9AppStream8MetadataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AppStream::Metadata",
        "FormatStyle",
        "FormatStyleUnknown",
        "FormatStyleMetainfo",
        "FormatStyleCatalog",
        "FormatKind",
        "FormatKindUnknown",
        "FormatKindXml",
        "FormatKindYaml",
        "FormatKindDesktopEntry",
        "FormatVersion",
        "FormatVersionUnknown",
        "FormatVersionV10",
        "FormatVersionV12",
        "MetadataError",
        "MetadataErrorNoError",
        "MetadataErrorFailed",
        "MetadataErrorParse",
        "MetadataErrorFormatUnexpected",
        "MetadataErrorNoComponent",
        "MetadataErrorValueMissing"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'FormatStyle'
        QtMocHelpers::EnumData<enum FormatStyle>(1, 1, QMC::EnumFlags{}).add({
            {    2, FormatStyle::FormatStyleUnknown },
            {    3, FormatStyle::FormatStyleMetainfo },
            {    4, FormatStyle::FormatStyleCatalog },
        }),
        // enum 'FormatKind'
        QtMocHelpers::EnumData<enum FormatKind>(5, 5, QMC::EnumFlags{}).add({
            {    6, FormatKind::FormatKindUnknown },
            {    7, FormatKind::FormatKindXml },
            {    8, FormatKind::FormatKindYaml },
            {    9, FormatKind::FormatKindDesktopEntry },
        }),
        // enum 'FormatVersion'
        QtMocHelpers::EnumData<enum FormatVersion>(10, 10, QMC::EnumFlags{}).add({
            {   11, FormatVersion::FormatVersionUnknown },
            {   12, FormatVersion::FormatVersionV10 },
            {   13, FormatVersion::FormatVersionV12 },
        }),
        // enum 'MetadataError'
        QtMocHelpers::EnumData<enum MetadataError>(14, 14, QMC::EnumFlags{}).add({
            {   15, MetadataError::MetadataErrorNoError },
            {   16, MetadataError::MetadataErrorFailed },
            {   17, MetadataError::MetadataErrorParse },
            {   18, MetadataError::MetadataErrorFormatUnexpected },
            {   19, MetadataError::MetadataErrorNoComponent },
            {   20, MetadataError::MetadataErrorValueMissing },
        }),
    };
    return QtMocHelpers::metaObjectData<Metadata, qt_meta_tag_ZN9AppStream8MetadataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AppStream::Metadata::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9AppStream8MetadataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9AppStream8MetadataE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9AppStream8MetadataE_t>.metaTypes,
    nullptr
} };

QT_WARNING_POP
