/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
 *
 * This file was automatically generated from SPDX metadata.
 */

#pragma once

/* DO NOT EDIT THIS FILE MANUALLY! */

/* Data generated from SPDX list versions:
 *   Licenses:   568a84d
 *   Exceptions: 568a84d
 */

#include <glib/gi18n-lib.h>
#include "as-macros-private.h"

AS_BEGIN_PRIVATE_DECLS
/* clang-format off */

typedef struct {
	const gchar	*id;
	const gchar	*name;
	gboolean	is_floss;
} AsSpdxLicenseInfo;

AsSpdxLicenseInfo as_spdx_license_info_list[] = {
	{ "0BSD", "BSD Zero Clause License", TRUE },
	{ "3D-Slicer-1.0", "3D Slicer License v1.0", FALSE },
	{ "AAL", "Attribution Assurance License", TRUE },
	{ "ADSL", "Amazon Digital Services License", FALSE },
	{ "AFL-1.1", "Academic Free License v1.1", TRUE },
	{ "AFL-1.2", "Academic Free License v1.2", TRUE },
	{ "AFL-2.0", "Academic Free License v2.0", TRUE },
	{ "AFL-2.1", "Academic Free License v2.1", TRUE },
	{ "AFL-3.0", "Academic Free License v3.0", TRUE },
	{ "AGPL-1.0", "Affero General Public License v1.0", TRUE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "AGPL-1.0-only", N_("Affero General Public License v1.0 only"), FALSE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "AGPL-1.0-or-later", N_("Affero General Public License v1.0 or later"), FALSE },
	{ "AGPL-3.0", "GNU Affero General Public License v3.0", TRUE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "AGPL-3.0-only", N_("GNU Affero General Public License v3.0 only"), TRUE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "AGPL-3.0-or-later", N_("GNU Affero General Public License v3.0 or later"), TRUE },
	{ "ALGLIB-Documentation", "ALGLIB Documentation License", TRUE },
	{ "AMD-newlib", "AMD newlib License", FALSE },
	{ "AMDPLPA", "AMD's plpa_map.c License", FALSE },
	{ "AML", "Apple MIT License", FALSE },
	{ "AML-glslang", "AML glslang variant License", FALSE },
	{ "AMPAS", "Academy of Motion Picture Arts and Sciences BSD", FALSE },
	{ "ANTLR-PD", "ANTLR Software Rights Notice", FALSE },
	{ "ANTLR-PD-fallback", "ANTLR Software Rights Notice with license fallback", FALSE },
	{ "APAFML", "Adobe Postscript AFM License", FALSE },
	{ "APL-1.0", "Adaptive Public License 1.0", TRUE },
	{ "APSL-1.0", "Apple Public Source License 1.0", TRUE },
	{ "APSL-1.1", "Apple Public Source License 1.1", TRUE },
	{ "APSL-1.2", "Apple Public Source License 1.2", TRUE },
	{ "APSL-2.0", "Apple Public Source License 2.0", TRUE },
	{ "ASWF-Digital-Assets-1.0", "ASWF Digital Assets License version 1.0", FALSE },
	{ "ASWF-Digital-Assets-1.1", "ASWF Digital Assets License 1.1", FALSE },
	{ "Abstyles", "Abstyles License", FALSE },
	{ "AdaCore-doc", "AdaCore Doc License", FALSE },
	{ "Adobe-2006", "Adobe Systems Incorporated Source Code License Agreement", FALSE },
	{ "Adobe-Display-PostScript", "Adobe Display PostScript License", FALSE },
	{ "Adobe-Glyph", "Adobe Glyph List License", FALSE },
	{ "Adobe-Utopia", "Adobe Utopia Font License", FALSE },
	{ "Advanced-Cryptics-Dictionary", "Advanced Cryptics Dictionary License", FALSE },
	{ "Afmparse", "Afmparse License", FALSE },
	{ "Aladdin", "Aladdin Free Public License", FALSE },
	{ "Apache-1.0", "Apache License 1.0", TRUE },
	{ "Apache-1.1", "Apache License 1.1", TRUE },
	{ "Apache-2.0", "Apache License 2.0", TRUE },
	{ "App-s2p", "App::s2p License", FALSE },
	{ "Arphic-1999", "Arphic Public License", FALSE },
	{ "Artistic-1.0", "Artistic License 1.0", TRUE },
	{ "Artistic-1.0-Perl", "Artistic License 1.0 (Perl)", TRUE },
	{ "Artistic-1.0-cl8", "Artistic License 1.0 w/clause 8", TRUE },
	{ "Artistic-2.0", "Artistic License 2.0", TRUE },
	{ "Artistic-dist", "Artistic License 1.0 (dist)", FALSE },
	{ "Aspell-RU", "Aspell Russian License", FALSE },
	{ "BOLA-1.1", "Buena Onda License Agreement v1.1", FALSE },
	{ "BSD-1-Clause", "BSD 1-Clause License", TRUE },
	{ "BSD-2-Clause", "BSD 2-Clause \"Simplified\" License", TRUE },
	{ "BSD-2-Clause-Darwin", "BSD 2-Clause - Ian Darwin variant", FALSE },
	{ "BSD-2-Clause-FreeBSD", "BSD 2-Clause FreeBSD License", TRUE },
	{ "BSD-2-Clause-NetBSD", "BSD 2-Clause NetBSD License", TRUE },
	{ "BSD-2-Clause-Patent", "BSD-2-Clause Plus Patent License", TRUE },
	{ "BSD-2-Clause-Views", "BSD 2-Clause with views sentence", FALSE },
	{ "BSD-2-Clause-first-lines", "BSD 2-Clause - first lines requirement", FALSE },
	{ "BSD-2-Clause-pkgconf-disclaimer", "BSD 2-Clause pkgconf disclaimer variant", FALSE },
	{ "BSD-3-Clause", "BSD 3-Clause \"New\" or \"Revised\" License", TRUE },
	{ "BSD-3-Clause-Attribution", "BSD with attribution", FALSE },
	{ "BSD-3-Clause-Clear", "BSD 3-Clause Clear License", TRUE },
	{ "BSD-3-Clause-HP", "Hewlett-Packard BSD variant license", FALSE },
	{ "BSD-3-Clause-LBNL", "Lawrence Berkeley National Labs BSD variant license", TRUE },
	{ "BSD-3-Clause-Modification", "BSD 3-Clause Modification", FALSE },
	{ "BSD-3-Clause-No-Military-License", "BSD 3-Clause No Military License", FALSE },
	{ "BSD-3-Clause-No-Nuclear-License", "BSD 3-Clause No Nuclear License", FALSE },
	{ "BSD-3-Clause-No-Nuclear-License-2014", "BSD 3-Clause No Nuclear License 2014", FALSE },
	{ "BSD-3-Clause-No-Nuclear-Warranty", "BSD 3-Clause No Nuclear Warranty", FALSE },
	{ "BSD-3-Clause-Open-MPI", "BSD 3-Clause Open MPI variant", TRUE },
	{ "BSD-3-Clause-Sun", "BSD 3-Clause Sun Microsystems", FALSE },
	{ "BSD-3-Clause-Tso", "BSD 3-Clause Tso variant", FALSE },
	{ "BSD-3-Clause-acpica", "BSD 3-Clause acpica variant", FALSE },
	{ "BSD-3-Clause-flex", "BSD 3-Clause Flex variant", FALSE },
	{ "BSD-4-Clause", "BSD 4-Clause \"Original\" or \"Old\" License", TRUE },
	{ "BSD-4-Clause-Shortened", "BSD 4 Clause Shortened", FALSE },
	{ "BSD-4-Clause-UC", "BSD-4-Clause (University of California-Specific)", FALSE },
	{ "BSD-4.3RENO", "BSD 4.3 RENO License", FALSE },
	{ "BSD-4.3TAHOE", "BSD 4.3 TAHOE License", FALSE },
	{ "BSD-Advertising-Acknowledgement", "BSD Advertising Acknowledgement License", FALSE },
	{ "BSD-Attribution-HPND-disclaimer", "BSD with Attribution and HPND disclaimer", FALSE },
	{ "BSD-Inferno-Nettverk", "BSD-Inferno-Nettverk", FALSE },
	{ "BSD-Mark-Modifications", "BSD Mark Modifications License", FALSE },
	{ "BSD-Protection", "BSD Protection License", FALSE },
	{ "BSD-Source-Code", "BSD Source Code Attribution", FALSE },
	{ "BSD-Source-Code-no-disclaimer", "BSD Source Code Attribution - no disclaimer", FALSE },
	{ "BSD-Source-beginning-file", "BSD Source Code Attribution - beginning of file variant", FALSE },
	{ "BSD-Systemics", "Systemics BSD variant license", FALSE },
	{ "BSD-Systemics-W3Works", "Systemics W3Works BSD variant license", FALSE },
	{ "BSL-1.0", "Boost Software License 1.0", TRUE },
	{ "BUSL-1.1", "Business Source License 1.1", FALSE },
	{ "Baekmuk", "Baekmuk License", FALSE },
	{ "Bahyph", "Bahyph License", FALSE },
	{ "Barr", "Barr License", FALSE },
	{ "Beerware", "Beerware License", FALSE },
	{ "BitTorrent-1.0", "BitTorrent Open Source License v1.0", FALSE },
	{ "BitTorrent-1.1", "BitTorrent Open Source License v1.1", TRUE },
	{ "Bitstream-Charter", "Bitstream Charter Font License", FALSE },
	{ "Bitstream-Vera", "Bitstream Vera Font License", FALSE },
	{ "BlueOak-1.0.0", "Blue Oak Model License 1.0.0", TRUE },
	{ "Boehm-GC", "Boehm-Demers-Weiser GC License", FALSE },
	{ "Boehm-GC-without-fee", "Boehm-Demers-Weiser GC License (without fee)", FALSE },
	{ "Borceux", "Borceux license", FALSE },
	{ "Brian-Gladman-2-Clause", "Brian Gladman 2-Clause License", FALSE },
	{ "Brian-Gladman-3-Clause", "Brian Gladman 3-Clause License", FALSE },
	{ "Brian-Gladman-3-Clause-no-conversion", "Brian Gladman 3-Clause License (no conversion clause)", FALSE },
	{ "Buddy", "Buddy License", FALSE },
	{ "Bugroff", "Bugroff License", FALSE },
	{ "C-UDA-1.0", "Computational Use of Data Agreement v1.0", FALSE },
	{ "CAL-1.0", "Cryptographic Autonomy License 1.0", TRUE },
	{ "CAL-1.0-Combined-Work-Exception", "Cryptographic Autonomy License 1.0 (Combined Work Exception)", TRUE },
	{ "CAPEC-tou", "Common Attack    Pattern Enumeration and Classification License", FALSE },
	{ "CATOSL-1.1", "Computer Associates Trusted Open Source License 1.1", TRUE },
	{ "CC-BY-1.0", "Creative Commons Attribution 1.0 Generic", FALSE },
	{ "CC-BY-2.0", "Creative Commons Attribution 2.0 Generic", FALSE },
	{ "CC-BY-2.5", "Creative Commons Attribution 2.5 Generic", FALSE },
	{ "CC-BY-2.5-AU", "Creative Commons Attribution 2.5 Australia", FALSE },
	{ "CC-BY-3.0", "Creative Commons Attribution 3.0 Unported", TRUE },
	{ "CC-BY-3.0-AT", "Creative Commons Attribution 3.0 Austria", TRUE },
	{ "CC-BY-3.0-AU", "Creative Commons Attribution 3.0 Australia", FALSE },
	{ "CC-BY-3.0-DE", "Creative Commons Attribution 3.0 Germany", TRUE },
	{ "CC-BY-3.0-IGO", "Creative Commons Attribution 3.0 IGO", FALSE },
	{ "CC-BY-3.0-NL", "Creative Commons Attribution 3.0 Netherlands", TRUE },
	{ "CC-BY-3.0-US", "Creative Commons Attribution 3.0 United States", TRUE },
	{ "CC-BY-4.0", "Creative Commons Attribution 4.0 International", TRUE },
	{ "CC-BY-NC-1.0", "Creative Commons Attribution Non Commercial 1.0 Generic", FALSE },
	{ "CC-BY-NC-2.0", "Creative Commons Attribution Non Commercial 2.0 Generic", FALSE },
	{ "CC-BY-NC-2.5", "Creative Commons Attribution Non Commercial 2.5 Generic", FALSE },
	{ "CC-BY-NC-3.0", "Creative Commons Attribution Non Commercial 3.0 Unported", FALSE },
	{ "CC-BY-NC-3.0-DE", "Creative Commons Attribution Non Commercial 3.0 Germany", FALSE },
	{ "CC-BY-NC-3.0-IGO", "Creative Commons Attribution Non Commercial 3.0 IGO", FALSE },
	{ "CC-BY-NC-4.0", "Creative Commons Attribution Non Commercial 4.0 International", FALSE },
	{ "CC-BY-NC-ND-1.0", "Creative Commons Attribution Non Commercial No Derivatives 1.0 Generic", FALSE },
	{ "CC-BY-NC-ND-2.0", "Creative Commons Attribution Non Commercial No Derivatives 2.0 Generic", FALSE },
	{ "CC-BY-NC-ND-2.5", "Creative Commons Attribution Non Commercial No Derivatives 2.5 Generic", FALSE },
	{ "CC-BY-NC-ND-3.0", "Creative Commons Attribution Non Commercial No Derivatives 3.0 Unported", FALSE },
	{ "CC-BY-NC-ND-3.0-DE", "Creative Commons Attribution Non Commercial No Derivatives 3.0 Germany", FALSE },
	{ "CC-BY-NC-ND-3.0-IGO", "Creative Commons Attribution Non Commercial No Derivatives 3.0 IGO", FALSE },
	{ "CC-BY-NC-ND-4.0", "Creative Commons Attribution Non Commercial No Derivatives 4.0 International", FALSE },
	{ "CC-BY-NC-SA-1.0", "Creative Commons Attribution Non Commercial Share Alike 1.0 Generic", FALSE },
	{ "CC-BY-NC-SA-2.0", "Creative Commons Attribution Non Commercial Share Alike 2.0 Generic", FALSE },
	{ "CC-BY-NC-SA-2.0-DE", "Creative Commons Attribution Non Commercial Share Alike 2.0 Germany", FALSE },
	{ "CC-BY-NC-SA-2.0-FR", "Creative Commons Attribution-NonCommercial-ShareAlike 2.0 France", FALSE },
	{ "CC-BY-NC-SA-2.0-UK", "Creative Commons Attribution Non Commercial Share Alike 2.0 England and Wales", FALSE },
	{ "CC-BY-NC-SA-2.5", "Creative Commons Attribution Non Commercial Share Alike 2.5 Generic", FALSE },
	{ "CC-BY-NC-SA-3.0", "Creative Commons Attribution Non Commercial Share Alike 3.0 Unported", FALSE },
	{ "CC-BY-NC-SA-3.0-DE", "Creative Commons Attribution Non Commercial Share Alike 3.0 Germany", FALSE },
	{ "CC-BY-NC-SA-3.0-IGO", "Creative Commons Attribution Non Commercial Share Alike 3.0 IGO", FALSE },
	{ "CC-BY-NC-SA-4.0", "Creative Commons Attribution Non Commercial Share Alike 4.0 International", FALSE },
	{ "CC-BY-ND-1.0", "Creative Commons Attribution No Derivatives 1.0 Generic", FALSE },
	{ "CC-BY-ND-2.0", "Creative Commons Attribution No Derivatives 2.0 Generic", FALSE },
	{ "CC-BY-ND-2.5", "Creative Commons Attribution No Derivatives 2.5 Generic", FALSE },
	{ "CC-BY-ND-3.0", "Creative Commons Attribution No Derivatives 3.0 Unported", FALSE },
	{ "CC-BY-ND-3.0-DE", "Creative Commons Attribution No Derivatives 3.0 Germany", FALSE },
	{ "CC-BY-ND-4.0", "Creative Commons Attribution No Derivatives 4.0 International", FALSE },
	{ "CC-BY-SA-1.0", "Creative Commons Attribution Share Alike 1.0 Generic", FALSE },
	{ "CC-BY-SA-2.0", "Creative Commons Attribution Share Alike 2.0 Generic", FALSE },
	{ "CC-BY-SA-2.0-UK", "Creative Commons Attribution Share Alike 2.0 England and Wales", FALSE },
	{ "CC-BY-SA-2.1-JP", "Creative Commons Attribution Share Alike 2.1 Japan", FALSE },
	{ "CC-BY-SA-2.5", "Creative Commons Attribution Share Alike 2.5 Generic", FALSE },
	{ "CC-BY-SA-3.0", "Creative Commons Attribution Share Alike 3.0 Unported", TRUE },
	{ "CC-BY-SA-3.0-AT", "Creative Commons Attribution Share Alike 3.0 Austria", TRUE },
	{ "CC-BY-SA-3.0-DE", "Creative Commons Attribution Share Alike 3.0 Germany", TRUE },
	{ "CC-BY-SA-3.0-IGO", "Creative Commons Attribution-ShareAlike 3.0 IGO", FALSE },
	{ "CC-BY-SA-4.0", "Creative Commons Attribution Share Alike 4.0 International", TRUE },
	{ "CC-PDDC", "Creative Commons Public Domain Dedication and Certification", FALSE },
	{ "CC-PDM-1.0", "Creative    Commons Public Domain Mark 1.0 Universal", FALSE },
	{ "CC-SA-1.0", "Creative Commons Share Alike 1.0 Generic", FALSE },
	{ "CC0-1.0", "Creative Commons Zero v1.0 Universal", TRUE },
	{ "CDDL-1.0", "Common Development and Distribution License 1.0", TRUE },
	{ "CDDL-1.1", "Common Development and Distribution License 1.1", TRUE },
	{ "CDL-1.0", "Common Documentation License 1.0", FALSE },
	{ "CDLA-Permissive-1.0", "Community Data License Agreement Permissive 1.0", FALSE },
	{ "CDLA-Permissive-2.0", "Community Data License Agreement Permissive 2.0", FALSE },
	{ "CDLA-Sharing-1.0", "Community Data License Agreement Sharing 1.0", FALSE },
	{ "CECILL-1.0", "CeCILL Free Software License Agreement v1.0", FALSE },
	{ "CECILL-1.1", "CeCILL Free Software License Agreement v1.1", FALSE },
	{ "CECILL-2.0", "CeCILL Free Software License Agreement v2.0", TRUE },
	{ "CECILL-2.1", "CeCILL Free Software License Agreement v2.1", TRUE },
	{ "CECILL-B", "CeCILL-B Free Software License Agreement", TRUE },
	{ "CECILL-C", "CeCILL-C Free Software License Agreement", TRUE },
	{ "CERN-OHL-1.1", "CERN Open Hardware Licence v1.1", FALSE },
	{ "CERN-OHL-1.2", "CERN Open Hardware Licence v1.2", FALSE },
	{ "CERN-OHL-P-2.0", "CERN Open Hardware Licence Version 2 - Permissive", TRUE },
	{ "CERN-OHL-S-2.0", "CERN Open Hardware Licence Version 2 - Strongly Reciprocal", TRUE },
	{ "CERN-OHL-W-2.0", "CERN Open Hardware Licence Version 2 - Weakly Reciprocal", TRUE },
	{ "CFITSIO", "CFITSIO License", FALSE },
	{ "CMU-Mach", "CMU Mach License", FALSE },
	{ "CMU-Mach-nodoc", "CMU    Mach - no notices-in-documentation variant", FALSE },
	{ "CNRI-Jython", "CNRI Jython License", FALSE },
	{ "CNRI-Python", "CNRI Python License", TRUE },
	{ "CNRI-Python-GPL-Compatible", "CNRI Python Open Source GPL Compatible License Agreement", FALSE },
	{ "COIL-1.0", "Copyfree Open Innovation License", FALSE },
	{ "CPAL-1.0", "Common Public Attribution License 1.0", TRUE },
	{ "CPL-1.0", "Common Public License 1.0", TRUE },
	{ "CPOL-1.02", "Code Project Open License 1.02", FALSE },
	{ "CUA-OPL-1.0", "CUA Office Public License v1.0", TRUE },
	{ "Caldera", "Caldera License", FALSE },
	{ "Caldera-no-preamble", "Caldera License (without preamble)", FALSE },
	{ "Catharon", "Catharon License", FALSE },
	{ "ClArtistic", "Clarified Artistic License", TRUE },
	{ "Clips", "Clips License", FALSE },
	{ "Community-Spec-1.0", "Community Specification License 1.0", FALSE },
	{ "Condor-1.1", "Condor Public License v1.1", TRUE },
	{ "Cornell-Lossless-JPEG", "Cornell Lossless JPEG License", FALSE },
	{ "Cronyx", "Cronyx License", FALSE },
	{ "Crossword", "Crossword License", FALSE },
	{ "CryptoSwift", "CryptoSwift License", FALSE },
	{ "CrystalStacker", "CrystalStacker License", FALSE },
	{ "Cube", "Cube License", FALSE },
	{ "D-FSL-1.0", "Deutsche Freie Software Lizenz", FALSE },
	{ "DEC-3-Clause", "DEC 3-Clause License", FALSE },
	{ "DL-DE-BY-2.0", "Data licence Germany – attribution – version 2.0", FALSE },
	{ "DL-DE-ZERO-2.0", "Data licence Germany – zero – version 2.0", FALSE },
	{ "DOC", "DOC License", FALSE },
	{ "DRL-1.0", "Detection Rule License 1.0", FALSE },
	{ "DRL-1.1", "Detection Rule License 1.1", FALSE },
	{ "DSDP", "DSDP License", FALSE },
	{ "DocBook-DTD", "DocBook DTD License", FALSE },
	{ "DocBook-Schema", "DocBook Schema License", FALSE },
	{ "DocBook-Stylesheet", "DocBook Stylesheet License", FALSE },
	{ "DocBook-XML", "DocBook XML License", FALSE },
	{ "Dotseqn", "Dotseqn License", FALSE },
	{ "ECL-1.0", "Educational Community License v1.0", TRUE },
	{ "ECL-2.0", "Educational Community License v2.0", TRUE },
	{ "EFL-1.0", "Eiffel Forum License v1.0", TRUE },
	{ "EFL-2.0", "Eiffel Forum License v2.0", TRUE },
	{ "EPICS", "EPICS Open License", FALSE },
	{ "EPL-1.0", "Eclipse Public License 1.0", TRUE },
	{ "EPL-2.0", "Eclipse Public License 2.0", TRUE },
	{ "ESA-PL-permissive-2.4", "European Space Agency Public License – v2.4 – Permissive (Type 3)", FALSE },
	{ "ESA-PL-strong-copyleft-2.4", "European Space Agency Public License (ESA-PL) - V2.4 - Strong Copyleft (Type 1)", FALSE },
	{ "ESA-PL-weak-copyleft-2.4", "European Space Agency Public License – v2.4 – Weak Copyleft (Type 2)", FALSE },
	{ "EUDatagrid", "EU DataGrid Software License", TRUE },
	{ "EUPL-1.0", "European Union Public License 1.0", FALSE },
	{ "EUPL-1.1", "European Union Public License 1.1", TRUE },
	{ "EUPL-1.2", "European Union Public License 1.2", TRUE },
	{ "Elastic-2.0", "Elastic License 2.0", FALSE },
	{ "Entessa", "Entessa Public License v1.0", TRUE },
	{ "ErlPL-1.1", "Erlang Public License v1.1", FALSE },
	{ "Eurosym", "Eurosym License", FALSE },
	{ "FBM", "Fuzzy Bitmap License", FALSE },
	{ "FDK-AAC", "Fraunhofer FDK AAC Codec Library", FALSE },
	{ "FDK-MPEG-H", "Fraunhofer FDK MPEG-H Software", FALSE },
	{ "FSFAP", "FSF All Permissive License", TRUE },
	{ "FSFAP-no-warranty-disclaimer", "FSF All Permissive License (without Warranty)", FALSE },
	{ "FSFUL", "FSF Unlimited License", FALSE },
	{ "FSFULLR", "FSF Unlimited License (with License Retention)", FALSE },
	{ "FSFULLRSD", "FSF Unlimited License (with License Retention and Short Disclaimer)", FALSE },
	{ "FSFULLRWD", "FSF Unlimited License (With License Retention and Warranty Disclaimer)", FALSE },
	{ "FSL-1.1-ALv2", "Functional Source License, Version 1.1, ALv2 Future License", FALSE },
	{ "FSL-1.1-MIT", "Functional Source License, Version 1.1, MIT Future License", FALSE },
	{ "FTL", "Freetype Project License", TRUE },
	{ "Fair", "Fair License", TRUE },
	{ "Ferguson-Twofish", "Ferguson Twofish License", FALSE },
	{ "Frameworx-1.0", "Frameworx Open License 1.0", TRUE },
	{ "FreeBSD-DOC", "FreeBSD Documentation License", FALSE },
	{ "FreeImage", "FreeImage Public License v1.0", FALSE },
	{ "Furuseth", "Furuseth License", FALSE },
	{ "GCR-docs", "Gnome GCR Documentation License", FALSE },
	{ "GD", "GD License", FALSE },
	{ "GFDL-1.1", "GNU Free Documentation License v1.1", TRUE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "GFDL-1.1-invariants-only", N_("GNU Free Documentation License v1.1 only - invariants"), FALSE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "GFDL-1.1-invariants-or-later", N_("GNU Free Documentation License v1.1 or later - invariants"), FALSE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "GFDL-1.1-no-invariants-only", N_("GNU Free Documentation License v1.1 only - no invariants"), FALSE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "GFDL-1.1-no-invariants-or-later", N_("GNU Free Documentation License v1.1 or later - no invariants"), FALSE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "GFDL-1.1-only", N_("GNU Free Documentation License v1.1 only"), TRUE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "GFDL-1.1-or-later", N_("GNU Free Documentation License v1.1 or later"), TRUE },
	{ "GFDL-1.2", "GNU Free Documentation License v1.2", TRUE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "GFDL-1.2-invariants-only", N_("GNU Free Documentation License v1.2 only - invariants"), FALSE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "GFDL-1.2-invariants-or-later", N_("GNU Free Documentation License v1.2 or later - invariants"), FALSE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "GFDL-1.2-no-invariants-only", N_("GNU Free Documentation License v1.2 only - no invariants"), FALSE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "GFDL-1.2-no-invariants-or-later", N_("GNU Free Documentation License v1.2 or later - no invariants"), FALSE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "GFDL-1.2-only", N_("GNU Free Documentation License v1.2 only"), TRUE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "GFDL-1.2-or-later", N_("GNU Free Documentation License v1.2 or later"), TRUE },
	{ "GFDL-1.3", "GNU Free Documentation License v1.3", TRUE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "GFDL-1.3-invariants-only", N_("GNU Free Documentation License v1.3 only - invariants"), FALSE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "GFDL-1.3-invariants-or-later", N_("GNU Free Documentation License v1.3 or later - invariants"), FALSE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "GFDL-1.3-no-invariants-only", N_("GNU Free Documentation License v1.3 only - no invariants"), FALSE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "GFDL-1.3-no-invariants-or-later", N_("GNU Free Documentation License v1.3 or later - no invariants"), FALSE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "GFDL-1.3-only", N_("GNU Free Documentation License v1.3 only"), TRUE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "GFDL-1.3-or-later", N_("GNU Free Documentation License v1.3 or later"), TRUE },
	{ "GL2PS", "GL2PS License", FALSE },
	{ "GLWTPL", "Good Luck With That Public License", FALSE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "GPL-1.0", N_("GNU General Public License v1.0 only"), FALSE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "GPL-1.0+", N_("GNU General Public License v1.0 or later"), FALSE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "GPL-1.0-only", N_("GNU General Public License v1.0 only"), FALSE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "GPL-1.0-or-later", N_("GNU General Public License v1.0 or later"), FALSE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "GPL-2.0", N_("GNU General Public License v2.0 only"), TRUE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "GPL-2.0+", N_("GNU General Public License v2.0 or later"), TRUE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "GPL-2.0-only", N_("GNU General Public License v2.0 only"), TRUE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "GPL-2.0-or-later", N_("GNU General Public License v2.0 or later"), TRUE },
	{ "GPL-2.0-with-GCC-exception", "GNU General Public License v2.0 w/GCC Runtime Library exception", FALSE },
	{ "GPL-2.0-with-autoconf-exception", "GNU General Public License v2.0 w/Autoconf exception", FALSE },
	{ "GPL-2.0-with-bison-exception", "GNU General Public License v2.0 w/Bison exception", FALSE },
	{ "GPL-2.0-with-classpath-exception", "GNU General Public License v2.0 w/Classpath exception", FALSE },
	{ "GPL-2.0-with-font-exception", "GNU General Public License v2.0 w/Font exception", FALSE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "GPL-3.0", N_("GNU General Public License v3.0 only"), TRUE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "GPL-3.0+", N_("GNU General Public License v3.0 or later"), TRUE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "GPL-3.0-only", N_("GNU General Public License v3.0 only"), TRUE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "GPL-3.0-or-later", N_("GNU General Public License v3.0 or later"), TRUE },
	{ "GPL-3.0-with-GCC-exception", "GNU General Public License v3.0 w/GCC Runtime Library exception", TRUE },
	{ "GPL-3.0-with-autoconf-exception", "GNU General Public License v3.0 w/Autoconf exception", FALSE },
	{ "Game-Programming-Gems", "Game Programming Gems License", FALSE },
	{ "Giftware", "Giftware License", FALSE },
	{ "Glide", "3dfx Glide License", FALSE },
	{ "Glulxe", "Glulxe License", FALSE },
	{ "Graphics-Gems", "Graphics Gems License", FALSE },
	{ "Gutmann", "Gutmann License", FALSE },
	{ "HDF5", "HDF5 License", FALSE },
	{ "HIDAPI", "HIDAPI License", FALSE },
	{ "HP-1986", "Hewlett-Packard 1986 License", FALSE },
	{ "HP-1989", "Hewlett-Packard 1989 License", FALSE },
	{ "HPND", "Historical Permission Notice and Disclaimer", TRUE },
	{ "HPND-DEC", "Historical Permission Notice and Disclaimer - DEC variant", FALSE },
	{ "HPND-Fenneberg-Livingston", "Historical Permission Notice and Disclaimer - Fenneberg-Livingston variant", FALSE },
	{ "HPND-INRIA-IMAG", "Historical Permission Notice and Disclaimer    - INRIA-IMAG variant", FALSE },
	{ "HPND-Intel", "Historical Permission Notice and Disclaimer - Intel variant", FALSE },
	{ "HPND-Kevlin-Henney", "Historical Permission Notice and Disclaimer - Kevlin Henney variant", FALSE },
	{ "HPND-MIT-disclaimer", "Historical Permission Notice and Disclaimer with MIT disclaimer", FALSE },
	{ "HPND-Markus-Kuhn", "Historical Permission Notice and Disclaimer - Markus Kuhn variant", FALSE },
	{ "HPND-Netrek", "Historical Permission Notice and Disclaimer - Netrek variant", FALSE },
	{ "HPND-Pbmplus", "Historical Permission Notice and Disclaimer - Pbmplus variant", FALSE },
	{ "HPND-SMC", "Historical Permission Notice and Disclaimer - SMC variant", FALSE },
	{ "HPND-UC", "Historical Permission Notice and Disclaimer - University of California variant", FALSE },
	{ "HPND-UC-export-US", "Historical Permission Notice and Disclaimer - University of California, US export warning", FALSE },
	{ "HPND-doc", "Historical Permission Notice and Disclaimer - documentation variant", FALSE },
	{ "HPND-doc-sell", "Historical Permission Notice and Disclaimer - documentation sell variant", FALSE },
	{ "HPND-export-US", "HPND with US Government export control warning", FALSE },
	{ "HPND-export-US-acknowledgement", "HPND with US Government export control warning and acknowledgment", FALSE },
	{ "HPND-export-US-modify", "HPND with US Government export control warning and modification rqmt", FALSE },
	{ "HPND-export2-US", "HPND with US Government export control and 2 disclaimers", FALSE },
	{ "HPND-merchantability-variant", "Historical Permission Notice and Disclaimer - merchantability variant", FALSE },
	{ "HPND-sell-MIT-disclaimer-xserver", "Historical Permission Notice and Disclaimer - sell xserver variant with MIT disclaimer", FALSE },
	{ "HPND-sell-regexpr", "Historical Permission Notice and Disclaimer - sell regexpr variant", FALSE },
	{ "HPND-sell-variant", "Historical Permission Notice and Disclaimer - sell variant", FALSE },
	{ "HPND-sell-variant-MIT-disclaimer", "HPND sell variant with MIT disclaimer", FALSE },
	{ "HPND-sell-variant-MIT-disclaimer-rev", "HPND sell variant with MIT disclaimer - reverse", FALSE },
	{ "HPND-sell-variant-critical-systems", "HPND - sell variant with safety critical systems clause", FALSE },
	{ "HTMLTIDY", "HTML Tidy License", FALSE },
	{ "HaskellReport", "Haskell Language Report License", FALSE },
	{ "Hippocratic-2.1", "Hippocratic License 2.1", FALSE },
	{ "IBM-pibs", "IBM PowerPC Initialization and Boot Software", FALSE },
	{ "ICU", "ICU License", TRUE },
	{ "IEC-Code-Components-EULA", "IEC    Code Components End-user licence agreement", FALSE },
	{ "IJG", "Independent JPEG Group License", TRUE },
	{ "IJG-short", "Independent JPEG Group License - short", FALSE },
	{ "IPA", "IPA Font License", TRUE },
	{ "IPL-1.0", "IBM Public License v1.0", TRUE },
	{ "ISC", "ISC License", TRUE },
	{ "ISC-Veillard", "ISC Veillard variant", FALSE },
	{ "ISO-permission", "ISO permission notice", FALSE },
	{ "ImageMagick", "ImageMagick License", FALSE },
	{ "Imlib2", "Imlib2 License", TRUE },
	{ "Info-ZIP", "Info-ZIP License", FALSE },
	{ "Informatica", "Informatica License", FALSE },
	{ "Inner-Net-2.0", "Inner Net License v2.0", FALSE },
	{ "InnoSetup", "Inno Setup License", FALSE },
	{ "Intel", "Intel Open Source License", TRUE },
	{ "Intel-ACPI", "Intel ACPI Software License Agreement", FALSE },
	{ "Interbase-1.0", "Interbase Public License v1.0", FALSE },
	{ "JPL-image", "JPL Image Use Policy", FALSE },
	{ "JPNIC", "Japan Network Information Center License", FALSE },
	{ "JSON", "JSON License", FALSE },
	{ "Jam", "Jam License", TRUE },
	{ "JasPer-2.0", "JasPer License", FALSE },
	{ "Kastrup", "Kastrup License", FALSE },
	{ "Kazlib", "Kazlib License", FALSE },
	{ "Knuth-CTAN", "Knuth CTAN License", FALSE },
	{ "LAL-1.2", "Licence Art Libre 1.2", FALSE },
	{ "LAL-1.3", "Licence Art Libre 1.3", FALSE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "LGPL-2.0", N_("GNU Library General Public License v2 only"), TRUE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "LGPL-2.0+", N_("GNU Library General Public License v2 or later"), TRUE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "LGPL-2.0-only", N_("GNU Library General Public License v2 only"), TRUE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "LGPL-2.0-or-later", N_("GNU Library General Public License v2 or later"), TRUE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "LGPL-2.1", N_("GNU Lesser General Public License v2.1 only"), TRUE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "LGPL-2.1+", N_("GNU Lesser General Public License v2.1 or later"), TRUE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "LGPL-2.1-only", N_("GNU Lesser General Public License v2.1 only"), TRUE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "LGPL-2.1-or-later", N_("GNU Lesser General Public License v2.1 or later"), TRUE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "LGPL-3.0", N_("GNU Lesser General Public License v3.0 only"), TRUE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "LGPL-3.0+", N_("GNU Lesser General Public License v3.0 or later"), TRUE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "LGPL-3.0-only", N_("GNU Lesser General Public License v3.0 only"), TRUE },
	/* TRANSLATORS: Please do not translate the license name itself. */
	{ "LGPL-3.0-or-later", N_("GNU Lesser General Public License v3.0 or later"), TRUE },
	{ "LGPLLR", "Lesser General Public License For Linguistic Resources", FALSE },
	{ "LOOP", "Common Lisp LOOP License", FALSE },
	{ "LPD-document", "LPD Documentation License", FALSE },
	{ "LPL-1.0", "Lucent Public License Version 1.0", TRUE },
	{ "LPL-1.02", "Lucent Public License v1.02", TRUE },
	{ "LPPL-1.0", "LaTeX Project Public License v1.0", FALSE },
	{ "LPPL-1.1", "LaTeX Project Public License v1.1", FALSE },
	{ "LPPL-1.2", "LaTeX Project Public License v1.2", TRUE },
	{ "LPPL-1.3a", "LaTeX Project Public License v1.3a", TRUE },
	{ "LPPL-1.3c", "LaTeX Project Public License v1.3c", TRUE },
	{ "LZMA-SDK-9.11-to-9.20", "LZMA SDK License (versions 9.11 to 9.20)", FALSE },
	{ "LZMA-SDK-9.22", "LZMA SDK License (versions 9.22 and beyond)", FALSE },
	{ "Latex2e", "Latex2e License", FALSE },
	{ "Latex2e-translated-notice", "Latex2e with translated notice permission", FALSE },
	{ "Leptonica", "Leptonica License", FALSE },
	{ "LiLiQ-P-1.1", "Licence Libre du Québec – Permissive version 1.1", TRUE },
	{ "LiLiQ-R-1.1", "Licence Libre du Québec – Réciprocité version 1.1", TRUE },
	{ "LiLiQ-Rplus-1.1", "Licence Libre du Québec – Réciprocité forte version 1.1", TRUE },
	{ "Libpng", "libpng License", FALSE },
	{ "Linux-OpenIB", "Linux Kernel Variant of OpenIB.org license", FALSE },
	{ "Linux-man-pages-1-para", "Linux man-pages - 1 paragraph", FALSE },
	{ "Linux-man-pages-copyleft", "Linux man-pages Copyleft", FALSE },
	{ "Linux-man-pages-copyleft-2-para", "Linux man-pages Copyleft - 2 paragraphs", FALSE },
	{ "Linux-man-pages-copyleft-var", "Linux man-pages Copyleft Variant", FALSE },
	{ "Lucida-Bitmap-Fonts", "Lucida Bitmap Fonts License", FALSE },
	{ "MIPS", "MIPS License", FALSE },
	{ "MIT", "MIT License", TRUE },
	{ "MIT-0", "MIT No Attribution", TRUE },
	{ "MIT-CMU", "CMU License", FALSE },
	{ "MIT-Click", "MIT Click License", FALSE },
	{ "MIT-Festival", "MIT Festival Variant", FALSE },
	{ "MIT-Khronos-old", "MIT Khronos - old variant", FALSE },
	{ "MIT-Modern-Variant", "MIT License Modern Variant", TRUE },
	{ "MIT-STK", "MIT-STK License", FALSE },
	{ "MIT-Wu", "MIT Tom Wu Variant", FALSE },
	{ "MIT-advertising", "Enlightenment License (e16)", FALSE },
	{ "MIT-enna", "enna License", FALSE },
	{ "MIT-feh", "feh License", FALSE },
	{ "MIT-open-group", "MIT Open Group variant", FALSE },
	{ "MIT-testregex", "MIT testregex Variant", FALSE },
	{ "MITNFA", "MIT +no-false-attribs license", FALSE },
	{ "MMIXware", "MMIXware License", FALSE },
	{ "MMPL-1.0.1", "Minecraft Mod Public License v1.0.1", FALSE },
	{ "MPEG-SSG", "MPEG Software Simulation", FALSE },
	{ "MPL-1.0", "Mozilla Public License 1.0", TRUE },
	{ "MPL-1.1", "Mozilla Public License 1.1", TRUE },
	{ "MPL-2.0", "Mozilla Public License 2.0", TRUE },
	{ "MPL-2.0-no-copyleft-exception", "Mozilla Public License 2.0 (no copyleft exception)", TRUE },
	{ "MS-LPL", "Microsoft Limited Public License", FALSE },
	{ "MS-PL", "Microsoft Public License", TRUE },
	{ "MS-RL", "Microsoft Reciprocal License", TRUE },
	{ "MTLL", "Matrix Template Library License", FALSE },
	{ "MVT-1.1", "MVT License 1.1", FALSE },
	{ "Mackerras-3-Clause", "Mackerras 3-Clause License", FALSE },
	{ "Mackerras-3-Clause-acknowledgment", "Mackerras 3-Clause - acknowledgment variant", FALSE },
	{ "MakeIndex", "MakeIndex License", FALSE },
	{ "Martin-Birgmeier", "Martin Birgmeier License", FALSE },
	{ "McPhee-slideshow", "McPhee Slideshow License", FALSE },
	{ "Minpack", "Minpack License", FALSE },
	{ "MirOS", "The MirOS Licence", TRUE },
	{ "Motosoto", "Motosoto License", TRUE },
	{ "MulanPSL-1.0", "Mulan Permissive Software License, Version 1", FALSE },
	{ "MulanPSL-2.0", "Mulan Permissive Software License, Version 2", TRUE },
	{ "Multics", "Multics License", TRUE },
	{ "Mup", "Mup License", FALSE },
	{ "NAIST-2003", "Nara Institute of Science and Technology License (2003)", FALSE },
	{ "NASA-1.3", "NASA Open Source Agreement 1.3", TRUE },
	{ "NBPL-1.0", "Net Boolean Public License v1", FALSE },
	{ "NCBI-PD", "NCBI Public Domain Notice", FALSE },
	{ "NCGL-UK-2.0", "Non-Commercial Government Licence", FALSE },
	{ "NCL", "NCL Source Code License", FALSE },
	{ "NCSA", "University of Illinois/NCSA Open Source License", TRUE },
	{ "NGPL", "Nethack General Public License", TRUE },
	{ "NICTA-1.0", "NICTA Public Software License, Version 1.0", FALSE },
	{ "NIST-PD", "NIST Public Domain Notice", FALSE },
	{ "NIST-PD-TNT", "NIST    Public Domain Notice TNT variant", FALSE },
	{ "NIST-PD-fallback", "NIST Public Domain Notice with license fallback", FALSE },
	{ "NIST-Software", "NIST Software License", FALSE },
	{ "NLOD-1.0", "Norwegian Licence for Open Government Data (NLOD) 1.0", FALSE },
	{ "NLOD-2.0", "Norwegian Licence for Open Government Data (NLOD) 2.0", FALSE },
	{ "NLPL", "No Limit Public License", FALSE },
	{ "NOSL", "Netizen Open Source License", TRUE },
	{ "NPL-1.0", "Netscape Public License v1.0", TRUE },
	{ "NPL-1.1", "Netscape Public License v1.1", TRUE },
	{ "NPOSL-3.0", "Non-Profit Open Software License 3.0", TRUE },
	{ "NRL", "NRL License", FALSE },
	{ "NTIA-PD", "NTIA Public Domain Notice", FALSE },
	{ "NTP", "NTP License", TRUE },
	{ "NTP-0", "NTP No Attribution", FALSE },
	{ "Naumen", "Naumen Public License", TRUE },
	{ "Net-SNMP", "Net-SNMP License", FALSE },
	{ "NetCDF", "NetCDF license", FALSE },
	{ "Newsletr", "Newsletr License", FALSE },
	{ "Nokia", "Nokia Open Source License", TRUE },
	{ "Noweb", "Noweb License", FALSE },
	{ "Nunit", "Nunit License", TRUE },
	{ "O-UDA-1.0", "Open Use of Data Agreement v1.0", FALSE },
	{ "OAR", "OAR License", FALSE },
	{ "OCCT-PL", "Open CASCADE Technology Public License", FALSE },
	{ "OCLC-2.0", "OCLC Research Public License 2.0", TRUE },
	{ "ODC-By-1.0", "Open Data Commons Attribution License v1.0", FALSE },
	{ "ODbL-1.0", "Open Data Commons Open Database License v1.0", TRUE },
	{ "OFFIS", "OFFIS License", FALSE },
	{ "OFL-1.0", "SIL Open Font License 1.0", TRUE },
	{ "OFL-1.0-RFN", "SIL Open Font License 1.0 with Reserved Font Name", FALSE },
	{ "OFL-1.0-no-RFN", "SIL Open Font License 1.0 with no Reserved Font Name", FALSE },
	{ "OFL-1.1", "SIL Open Font License 1.1", TRUE },
	{ "OFL-1.1-RFN", "SIL Open Font License 1.1 with Reserved Font Name", TRUE },
	{ "OFL-1.1-no-RFN", "SIL Open Font License 1.1 with no Reserved Font Name", TRUE },
	{ "OGC-1.0", "OGC Software License, Version 1.0", FALSE },
	{ "OGDL-Taiwan-1.0", "Taiwan Open Government Data License, version 1.0", FALSE },
	{ "OGL-Canada-2.0", "Open Government Licence - Canada", FALSE },
	{ "OGL-UK-1.0", "Open Government Licence v1.0", FALSE },
	{ "OGL-UK-2.0", "Open Government Licence v2.0", FALSE },
	{ "OGL-UK-3.0", "Open Government Licence v3.0", FALSE },
	{ "OGTSL", "Open Group Test Suite License", TRUE },
	{ "OLDAP-1.1", "Open LDAP Public License v1.1", FALSE },
	{ "OLDAP-1.2", "Open LDAP Public License v1.2", FALSE },
	{ "OLDAP-1.3", "Open LDAP Public License v1.3", FALSE },
	{ "OLDAP-1.4", "Open LDAP Public License v1.4", FALSE },
	{ "OLDAP-2.0", "Open LDAP Public License v2.0 (or possibly 2.0A and 2.0B)", FALSE },
	{ "OLDAP-2.0.1", "Open LDAP Public License v2.0.1", FALSE },
	{ "OLDAP-2.1", "Open LDAP Public License v2.1", FALSE },
	{ "OLDAP-2.2", "Open LDAP Public License v2.2", FALSE },
	{ "OLDAP-2.2.1", "Open LDAP Public License v2.2.1", FALSE },
	{ "OLDAP-2.2.2", "Open LDAP Public License 2.2.2", FALSE },
	{ "OLDAP-2.3", "Open LDAP Public License v2.3", TRUE },
	{ "OLDAP-2.4", "Open LDAP Public License v2.4", FALSE },
	{ "OLDAP-2.5", "Open LDAP Public License v2.5", FALSE },
	{ "OLDAP-2.6", "Open LDAP Public License v2.6", FALSE },
	{ "OLDAP-2.7", "Open LDAP Public License v2.7", TRUE },
	{ "OLDAP-2.8", "Open LDAP Public License v2.8", TRUE },
	{ "OLFL-1.3", "Open Logistics Foundation License Version 1.3", TRUE },
	{ "OML", "Open Market License", FALSE },
	{ "OPL-1.0", "Open Public License v1.0", FALSE },
	{ "OPL-UK-3.0", "United    Kingdom Open Parliament Licence v3.0", FALSE },
	{ "OPUBL-1.0", "Open Publication License v1.0", FALSE },
	{ "OSC-1.0", "OSC License 1.0", TRUE },
	{ "OSET-PL-2.1", "OSET Public License version 2.1", TRUE },
	{ "OSL-1.0", "Open Software License 1.0", TRUE },
	{ "OSL-1.1", "Open Software License 1.1", TRUE },
	{ "OSL-2.0", "Open Software License 2.0", TRUE },
	{ "OSL-2.1", "Open Software License 2.1", TRUE },
	{ "OSL-3.0", "Open Software License 3.0", TRUE },
	{ "OSSP", "OSSP License", FALSE },
	{ "OpenMDW-1.0", "OpenMDW License Agreement v1.0", FALSE },
	{ "OpenPBS-2.3", "OpenPBS v2.3 Software License", FALSE },
	{ "OpenSSL", "OpenSSL License", TRUE },
	{ "OpenSSL-standalone", "OpenSSL License - standalone", FALSE },
	{ "OpenVision", "OpenVision License", FALSE },
	{ "PADL", "PADL License", FALSE },
	{ "PDDL-1.0", "Open Data Commons Public Domain Dedication & License 1.0", FALSE },
	{ "PHP-3.0", "PHP License v3.0", TRUE },
	{ "PHP-3.01", "PHP License v3.01", TRUE },
	{ "PPL", "Peer Production License", FALSE },
	{ "PSF-2.0", "Python Software Foundation License 2.0", TRUE },
	{ "ParaType-Free-Font-1.3", "ParaType Free Font Licensing Agreement v1.3", FALSE },
	{ "Parity-6.0.0", "The Parity Public License 6.0.0", FALSE },
	{ "Parity-7.0.0", "The Parity Public License 7.0.0", FALSE },
	{ "Pixar", "Pixar License", FALSE },
	{ "Plexus", "Plexus Classworlds License", FALSE },
	{ "PolyForm-Noncommercial-1.0.0", "PolyForm Noncommercial License 1.0.0", FALSE },
	{ "PolyForm-Small-Business-1.0.0", "PolyForm Small Business License 1.0.0", FALSE },
	{ "PostgreSQL", "PostgreSQL License", TRUE },
	{ "Python-2.0", "Python License 2.0", TRUE },
	{ "Python-2.0.1", "Python License 2.0.1", FALSE },
	{ "QPL-1.0", "Q Public License 1.0", TRUE },
	{ "QPL-1.0-INRIA-2004", "Q Public License 1.0 - INRIA 2004 variant", FALSE },
	{ "Qhull", "Qhull License", FALSE },
	{ "RHeCos-1.1", "Red Hat eCos Public License v1.1", FALSE },
	{ "RPL-1.1", "Reciprocal Public License 1.1", TRUE },
	{ "RPL-1.5", "Reciprocal Public License 1.5", TRUE },
	{ "RPSL-1.0", "RealNetworks Public Source License v1.0", TRUE },
	{ "RSA-MD", "RSA Message-Digest License", FALSE },
	{ "RSCPL", "Ricoh Source Code Public License", TRUE },
	{ "Rdisc", "Rdisc License", FALSE },
	{ "Ruby", "Ruby License", TRUE },
	{ "Ruby-pty", "Ruby pty extension license", FALSE },
	{ "SAX-PD", "Sax Public Domain Notice", FALSE },
	{ "SAX-PD-2.0", "Sax Public Domain Notice 2.0", FALSE },
	{ "SCEA", "SCEA Shared Source License", FALSE },
	{ "SGI-B-1.0", "SGI Free Software License B v1.0", FALSE },
	{ "SGI-B-1.1", "SGI Free Software License B v1.1", FALSE },
	{ "SGI-B-2.0", "SGI Free Software License B v2.0", TRUE },
	{ "SGI-OpenGL", "SGI OpenGL License", FALSE },
	{ "SGMLUG-PM", "SGMLUG Parser Materials License", FALSE },
	{ "SGP4", "SGP4 Permission Notice", FALSE },
	{ "SHL-0.5", "Solderpad Hardware License v0.5", FALSE },
	{ "SHL-0.51", "Solderpad Hardware License, Version 0.51", FALSE },
	{ "SISSL", "Sun Industry Standards Source License v1.1", TRUE },
	{ "SISSL-1.2", "Sun Industry Standards Source License v1.2", FALSE },
	{ "SL", "SL License", FALSE },
	{ "SMAIL-GPL", "SMAIL General Public License", FALSE },
	{ "SMLNJ", "Standard ML of New Jersey License", TRUE },
	{ "SMPPL", "Secure Messaging Protocol Public License", FALSE },
	{ "SNIA", "SNIA Public License 1.1", FALSE },
	{ "SOFA", "SOFA Software License", FALSE },
	{ "SPL-1.0", "Sun Public License v1.0", TRUE },
	{ "SSH-OpenSSH", "SSH OpenSSH license", FALSE },
	{ "SSH-short", "SSH short notice", FALSE },
	{ "SSLeay-standalone", "SSLeay License - standalone", FALSE },
	{ "SSPL-1.0", "Server Side Public License, v 1", FALSE },
	{ "SUL-1.0", "Sustainable Use License v1.0", FALSE },
	{ "SWL", "Scheme Widget Library (SWL) Software License Agreement", FALSE },
	{ "Saxpath", "Saxpath License", FALSE },
	{ "SchemeReport", "Scheme Language Report License", FALSE },
	{ "Sendmail", "Sendmail License", FALSE },
	{ "Sendmail-8.23", "Sendmail License 8.23", FALSE },
	{ "Sendmail-Open-Source-1.1", "Sendmail Open Source License v1.1", FALSE },
	{ "SimPL-2.0", "Simple Public License 2.0", TRUE },
	{ "Sleepycat", "Sleepycat License", TRUE },
	{ "Soundex", "Soundex License", FALSE },
	{ "Spencer-86", "Spencer License 86", FALSE },
	{ "Spencer-94", "Spencer License 94", FALSE },
	{ "Spencer-99", "Spencer License 99", FALSE },
	{ "StandardML-NJ", "Standard ML of New Jersey License", TRUE },
	{ "SugarCRM-1.1.3", "SugarCRM Public License v1.1.3", FALSE },
	{ "Sun-PPP", "Sun PPP License", FALSE },
	{ "Sun-PPP-2000", "Sun PPP License (2000)", FALSE },
	{ "SunPro", "SunPro License", FALSE },
	{ "Symlinks", "Symlinks License", FALSE },
	{ "TAPR-OHL-1.0", "TAPR Open Hardware License v1.0", FALSE },
	{ "TCL", "TCL/TK License", FALSE },
	{ "TCP-wrappers", "TCP Wrappers License", FALSE },
	{ "TGPPL-1.0", "Transitive Grace Period Public Licence 1.0", FALSE },
	{ "TMate", "TMate Open Source License", FALSE },
	{ "TORQUE-1.1", "TORQUE v2.5+ Software License v1.1", FALSE },
	{ "TOSL", "Trusster Open Source License", FALSE },
	{ "TPDL", "Time::ParseDate License", FALSE },
	{ "TPL-1.0", "THOR Public License 1.0", FALSE },
	{ "TTWL", "Text-Tabs+Wrap License", FALSE },
	{ "TTYP0", "TTYP0 License", FALSE },
	{ "TU-Berlin-1.0", "Technische Universitaet Berlin License 1.0", FALSE },
	{ "TU-Berlin-2.0", "Technische Universitaet Berlin License 2.0", FALSE },
	{ "TekHVC", "TekHVC License", FALSE },
	{ "TermReadKey", "TermReadKey License", FALSE },
	{ "ThirdEye", "ThirdEye License", FALSE },
	{ "TrustedQSL", "TrustedQSL License", FALSE },
	{ "UCAR", "UCAR License", FALSE },
	{ "UCL-1.0", "Upstream Compatibility License v1.0", TRUE },
	{ "UMich-Merit", "Michigan/Merit Networks License", FALSE },
	{ "UPL-1.0", "Universal Permissive License v1.0", TRUE },
	{ "URT-RLE", "Utah Raster Toolkit Run Length Encoded License", FALSE },
	{ "Ubuntu-font-1.0", "Ubuntu Font Licence v1.0", FALSE },
	{ "UnRAR", "UnRAR License", FALSE },
	{ "Unicode-3.0", "Unicode License v3", TRUE },
	{ "Unicode-DFS-2015", "Unicode License Agreement - Data Files and Software (2015)", FALSE },
	{ "Unicode-DFS-2016", "Unicode License Agreement - Data Files and Software (2016)", TRUE },
	{ "Unicode-TOU", "Unicode Terms of Use", FALSE },
	{ "UnixCrypt", "UnixCrypt License", FALSE },
	{ "Unlicense", "The Unlicense", TRUE },
	{ "Unlicense-libtelnet", "Unlicense - libtelnet variant", FALSE },
	{ "Unlicense-libwhirlpool", "Unlicense - libwhirlpool variant", FALSE },
	{ "VOSTROM", "VOSTROM Public License for Open Source", FALSE },
	{ "VSL-1.0", "Vovida Software License v1.0", TRUE },
	{ "Vim", "Vim License", TRUE },
	{ "Vixie-Cron", "Vixie Cron License", FALSE },
	{ "W3C", "W3C Software Notice and License (2002-12-31)", TRUE },
	{ "W3C-19980720", "W3C Software Notice and License (1998-07-20)", FALSE },
	{ "W3C-20150513", "W3C Software Notice and Document License (2015-05-13)", TRUE },
	{ "WTFNMFPL", "Do What The F*ck You Want To But It's Not My Fault Public License", FALSE },
	{ "WTFPL", "Do What The F*ck You Want To Public License", TRUE },
	{ "Watcom-1.0", "Sybase Open Watcom Public License 1.0", TRUE },
	{ "Widget-Workshop", "Widget Workshop License", FALSE },
	{ "WordNet", "WordNet License", TRUE },
	{ "Wsuipa", "Wsuipa License", FALSE },
	{ "X11", "X11 License", TRUE },
	{ "X11-distribute-modifications-variant", "X11 License Distribution Modification Variant", FALSE },
	{ "X11-no-permit-persons", "X11 no permit persons clause", FALSE },
	{ "X11-swapped", "X11 swapped final paragraphs", FALSE },
	{ "XFree86-1.1", "XFree86 License 1.1", TRUE },
	{ "XSkat", "XSkat License", FALSE },
	{ "Xdebug-1.03", "Xdebug License v 1.03", FALSE },
	{ "Xerox", "Xerox License", FALSE },
	{ "Xfig", "Xfig License", FALSE },
	{ "Xnet", "X.Net License", TRUE },
	{ "YPL-1.0", "Yahoo! Public License v1.0", FALSE },
	{ "YPL-1.1", "Yahoo! Public License v1.1", TRUE },
	{ "ZPL-1.1", "Zope Public License 1.1", FALSE },
	{ "ZPL-2.0", "Zope Public License 2.0", TRUE },
	{ "ZPL-2.1", "Zope Public License 2.1", TRUE },
	{ "Zed", "Zed License", FALSE },
	{ "Zeeff", "Zeeff License", FALSE },
	{ "Zend-2.0", "Zend License v2.0", TRUE },
	{ "Zimbra-1.3", "Zimbra Public License v1.3", TRUE },
	{ "Zimbra-1.4", "Zimbra Public License v1.4", FALSE },
	{ "Zlib", "zlib License", TRUE },
	{ "any-OSI", "Any OSI License", FALSE },
	{ "any-OSI-perl-modules", "Any OSI License - Perl Modules", FALSE },
	{ "atc-game", "atc Game License", FALSE },
	{ "bcrypt-Solar-Designer", "bcrypt Solar Designer License", FALSE },
	{ "blessing", "SQLite Blessing", FALSE },
	{ "bzip2-1.0.5", "bzip2 and libbzip2 License v1.0.5", FALSE },
	{ "bzip2-1.0.6", "bzip2 and libbzip2 License v1.0.6", FALSE },
	{ "check-cvs", "check-cvs License", FALSE },
	{ "checkmk", "Checkmk License", FALSE },
	{ "copyleft-next-0.3.0", "copyleft-next 0.3.0", FALSE },
	{ "copyleft-next-0.3.1", "copyleft-next 0.3.1", FALSE },
	{ "curl", "curl License", FALSE },
	{ "cve-tou", "Common Vulnerability Enumeration ToU License", FALSE },
	{ "diffmark", "diffmark license", FALSE },
	{ "dtoa", "David M. Gay dtoa License", FALSE },
	{ "dvipdfm", "dvipdfm License", FALSE },
	{ "eCos-2.0", "eCos license version 2.0", TRUE },
	{ "eGenix", "eGenix.com Public License 1.1.0", FALSE },
	{ "etalab-2.0", "Etalab Open License 2.0", FALSE },
	{ "fwlw", "fwlw License", FALSE },
	{ "gSOAP-1.3b", "gSOAP Public License v1.3b", FALSE },
	{ "generic-xts", "Generic XTS License", FALSE },
	{ "gnuplot", "gnuplot License", TRUE },
	{ "gtkbook", "gtkbook License", FALSE },
	{ "hdparm", "hdparm License", FALSE },
	{ "hyphen-bulgarian", "hyphen-bulgarian License", FALSE },
	{ "iMatix", "iMatix Standard Function Library Agreement", TRUE },
	{ "jove", "Jove License", FALSE },
	{ "libpng-1.6.35", "PNG Reference Library License v1 (for libpng 0.5 through 1.6.35)", FALSE },
	{ "libpng-2.0", "PNG Reference Library version 2", FALSE },
	{ "libselinux-1.0", "libselinux public domain notice", FALSE },
	{ "libtiff", "libtiff License", FALSE },
	{ "libutil-David-Nugent", "libutil David Nugent License", FALSE },
	{ "lsof", "lsof License", FALSE },
	{ "magaz", "magaz License", FALSE },
	{ "mailprio", "mailprio License", FALSE },
	{ "man2html", "man2html License", FALSE },
	{ "metamail", "metamail License", FALSE },
	{ "mpi-permissive", "mpi Permissive License", FALSE },
	{ "mpich2", "mpich2 License", FALSE },
	{ "mplus", "mplus Font License", FALSE },
	{ "ngrep", "ngrep License", FALSE },
	{ "pkgconf", "pkgconf License", FALSE },
	{ "pnmstitch", "pnmstitch License", FALSE },
	{ "psfrag", "psfrag License", FALSE },
	{ "psutils", "psutils License", FALSE },
	{ "python-ldap", "Python ldap License", FALSE },
	{ "radvd", "radvd License", FALSE },
	{ "snprintf", "snprintf License", FALSE },
	{ "softSurfer", "softSurfer License", FALSE },
	{ "ssh-keyscan", "ssh-keyscan License", FALSE },
	{ "swrule", "swrule License", FALSE },
	{ "threeparttable", "threeparttable License", FALSE },
	{ "ulem", "ulem License", FALSE },
	{ "w3m", "w3m License", FALSE },
	{ "wwl", "WWL License", FALSE },
	{ "wxWindows", "wxWindows Library License", TRUE },
	{ "xinetd", "xinetd License", TRUE },
	{ "xkeyboard-config-Zinoviev", "xkeyboard-config Zinoviev License", FALSE },
	{ "xlock", "xlock License", FALSE },
	{ "xpp", "XPP License", FALSE },
	{ "xzoom", "xzoom License", FALSE },
	{ "zlib-acknowledgement", "zlib/libpng License with Acknowledgement", FALSE },
	{ NULL, NULL, FALSE },
};

AsSpdxLicenseInfo as_spdx_exception_info_list[] = {
	{ "389-exception", "389 Directory Server Exception" },
	{ "Asterisk-exception", "Asterisk exception" },
	{ "Asterisk-linking-protocols-exception", "Asterisk linking protocols exception" },
	{ "Autoconf-exception-2.0", "Autoconf exception 2.0" },
	{ "Autoconf-exception-3.0", "Autoconf exception 3.0" },
	{ "Autoconf-exception-generic", "Autoconf generic exception" },
	{ "Autoconf-exception-generic-3.0", "Autoconf generic exception for GPL-3.0" },
	{ "Autoconf-exception-macro", "Autoconf macro exception" },
	{ "Bison-exception-1.24", "Bison exception 1.24" },
	{ "Bison-exception-2.2", "Bison exception 2.2" },
	{ "Bootloader-exception", "Bootloader Distribution Exception" },
	{ "CGAL-linking-exception", "CGAL Linking Exception" },
	{ "CLISP-exception-2.0", "CLISP exception 2.0" },
	{ "Classpath-exception-2.0", "Classpath exception 2.0" },
	{ "Classpath-exception-2.0-short", "Classpath exception 2.0 - short" },
	{ "DigiRule-FOSS-exception", "DigiRule FOSS License Exception" },
	{ "Digia-Qt-LGPL-exception-1.1", "Digia Qt LGPL Exception version 1.1" },
	{ "FLTK-exception", "FLTK exception" },
	{ "Fawkes-Runtime-exception", "Fawkes Runtime Exception" },
	{ "Font-exception-2.0", "Font exception 2.0" },
	{ "GCC-exception-2.0", "GCC Runtime Library exception 2.0" },
	{ "GCC-exception-2.0-note", "GCC    Runtime Library exception 2.0 - note variant" },
	{ "GCC-exception-3.1", "GCC Runtime Library exception 3.1" },
	{ "GNAT-exception", "GNAT exception" },
	{ "GNOME-examples-exception", "GNOME examples exception" },
	{ "GNU-compiler-exception", "GNU Compiler Exception" },
	{ "GPL-3.0-389-ds-base-exception", "GPL-3.0 389 DS Base Exception" },
	{ "GPL-3.0-interface-exception", "GPL-3.0 Interface Exception" },
	{ "GPL-3.0-linking-exception", "GPL-3.0 Linking Exception" },
	{ "GPL-3.0-linking-source-exception", "GPL-3.0 Linking Exception (with Corresponding Source)" },
	{ "GPL-CC-1.0", "GPL Cooperation Commitment 1.0" },
	{ "GStreamer-exception-2005", "GStreamer Exception (2005)" },
	{ "GStreamer-exception-2008", "GStreamer Exception (2008)" },
	{ "Gmsh-exception", "Gmsh exception" },
	{ "Google-Patent-WebM", "Google Additional IP Rights Grant (Patents) - WebM" },
	{ "Independent-modules-exception", "Independent Module Linking exception" },
	{ "KiCad-libraries-exception", "KiCad Libraries Exception" },
	{ "LGPL-3.0-linking-exception", "LGPL-3.0 Linking Exception" },
	{ "LLGPL", "LLGPL Preamble" },
	{ "LLVM-exception", "LLVM Exception" },
	{ "LZMA-exception", "LZMA exception" },
	{ "Libtool-exception", "Libtool Exception" },
	{ "Linux-syscall-note", "Linux Syscall Note" },
	{ "Nokia-Qt-exception-1.1", "Nokia Qt LGPL exception 1.1" },
	{ "OCCT-exception-1.0", "Open CASCADE Exception 1.0" },
	{ "OCaml-LGPL-linking-exception", "OCaml LGPL Linking Exception" },
	{ "OpenJDK-assembly-exception-1.0", "OpenJDK Assembly exception 1.0" },
	{ "PCRE2-exception", "PCRE2 exception" },
	{ "PS-or-PDF-font-exception-20170817", "PS/PDF font exception (2017-08-17)" },
	{ "QPL-1.0-INRIA-2004-exception", "INRIA QPL 1.0 2004 variant exception" },
	{ "Qt-GPL-exception-1.0", "Qt GPL exception 1.0" },
	{ "Qt-LGPL-exception-1.1", "Qt LGPL exception 1.1" },
	{ "Qwt-exception-1.0", "Qwt exception 1.0" },
	{ "RRDtool-FLOSS-exception-2.0", "RRDtool FLOSS exception 2.0" },
	{ "SANE-exception", "SANE Exception" },
	{ "SHL-2.0", "Solderpad Hardware License v2.0" },
	{ "SHL-2.1", "Solderpad Hardware License v2.1" },
	{ "SWI-exception", "SWI exception" },
	{ "Simple-Library-Usage-exception", "Simple Library Usage Exception" },
	{ "Spelling-Provider-LGPL-exception", "Spelling Provider LGPL exception" },
	{ "Swift-exception", "Swift Exception" },
	{ "Texinfo-exception", "Texinfo exception" },
	{ "UBDL-exception", "Unmodified Binary Distribution exception" },
	{ "Universal-FOSS-exception-1.0", "Universal FOSS Exception, Version 1.0" },
	{ "WxWindows-exception-3.1", "WxWindows Library Exception 3.1" },
	{ "cryptsetup-OpenSSL-exception", "cryptsetup OpenSSL exception" },
	{ "eCos-exception-2.0", "eCos exception 2.0" },
	{ "erlang-otp-linking-exception", "Erlang/OTP Linking Exception" },
	{ "fmt-exception", "fmt exception" },
	{ "freertos-exception-2.0", "FreeRTOS Exception 2.0" },
	{ "gnu-javamail-exception", "GNU JavaMail exception" },
	{ "harbour-exception", "harbour exception" },
	{ "i2p-gpl-java-exception", "i2p GPL+Java Exception" },
	{ "kvirc-openssl-exception", "kvirc OpenSSL Exception" },
	{ "libpri-OpenH323-exception", "libpri OpenH323 exception" },
	{ "mif-exception", "Macros and Inline Functions Exception" },
	{ "mxml-exception", "mxml Exception" },
	{ "openvpn-openssl-exception", "OpenVPN OpenSSL Exception" },
	{ "polyparse-exception", "Polyparse Exception" },
	{ "romic-exception", "Romic Exception" },
	{ "rsync-linking-exception", "rsync Linking Exception" },
	{ "sqlitestudio-OpenSSL-exception", "sqlitestudio OpenSSL exception" },
	{ "stunnel-exception", "stunnel Exception" },
	{ "u-boot-exception-2.0", "U-Boot exception 2.0" },
	{ "vsftpd-openssl-exception", "vsftpd OpenSSL exception" },
	{ "x11vnc-openssl-exception", "x11vnc OpenSSL Exception" },
	{ NULL, NULL },
};

/* clang-format on */
AS_END_PRIVATE_DECLS
