/****************************************************************************
** Meta object code from reading C++ file 'contentrating.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../appstream/qt/contentrating.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'contentrating.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9AppStream13ContentRatingE_t {};
} // unnamed namespace

template <> constexpr inline auto AppStream::ContentRating::qt_create_metaobjectdata<qt_meta_tag_ZN9AppStream13ContentRatingE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AppStream::ContentRating",
        "RatingValue",
        "RatingValueUnknown",
        "RatingValueNone",
        "RatingValueMild",
        "RatingValueModerate",
        "RatingValueIntense",
        "System",
        "SystemUnknown",
        "SystemIncaa",
        "SystemAcb",
        "SystemDjctq",
        "SystemGsrr",
        "SystemPegi",
        "SystemKavi",
        "SystemUsk",
        "SystemEsra",
        "SystemCero",
        "SystemOflcnz",
        "SystemRussia",
        "SystemMda",
        "SystemGrac",
        "SystemEsrb",
        "SystemIarc"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'RatingValue'
        QtMocHelpers::EnumData<enum RatingValue>(1, 1, QMC::EnumFlags{}).add({
            {    2, RatingValue::RatingValueUnknown },
            {    3, RatingValue::RatingValueNone },
            {    4, RatingValue::RatingValueMild },
            {    5, RatingValue::RatingValueModerate },
            {    6, RatingValue::RatingValueIntense },
        }),
        // enum 'System'
        QtMocHelpers::EnumData<enum System>(7, 7, QMC::EnumFlags{}).add({
            {    8, System::SystemUnknown },
            {    9, System::SystemIncaa },
            {   10, System::SystemAcb },
            {   11, System::SystemDjctq },
            {   12, System::SystemGsrr },
            {   13, System::SystemPegi },
            {   14, System::SystemKavi },
            {   15, System::SystemUsk },
            {   16, System::SystemEsra },
            {   17, System::SystemCero },
            {   18, System::SystemOflcnz },
            {   19, System::SystemRussia },
            {   20, System::SystemMda },
            {   21, System::SystemGrac },
            {   22, System::SystemEsrb },
            {   23, System::SystemIarc },
        }),
    };
    return QtMocHelpers::metaObjectData<ContentRating, qt_meta_tag_ZN9AppStream13ContentRatingE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AppStream::ContentRating::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9AppStream13ContentRatingE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9AppStream13ContentRatingE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9AppStream13ContentRatingE_t>.metaTypes,
    nullptr
} };

QT_WARNING_POP
