/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
 *
 * Copyright (C) 2025-2026 Matthias Klumpp <matthias@tenstral.net>
 *
 * Licensed under the GNU Lesser General Public License Version 2.1
 *
 * This library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the license, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <glib-object.h>

#include "asw-sandbox.h"

G_BEGIN_DECLS

#define ASW_TYPE_WORKER (asw_worker_get_type ())
G_DECLARE_FINAL_TYPE (AswWorker, asw_worker, ASW, WORKER, GObject)

AswWorker *asw_worker_new_for_fd (gint	   socket_fd,
				  GError **error);
gboolean   asw_worker_send_hello (AswWorker	       *worker,
				  const AswSandboxInfo *sandbox,
				  GError	      **error);
gint	   asw_worker_run (AswWorker *worker);

G_END_DECLS
