/****************************************************************************
** Meta object code from reading C++ file 'component.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../appstream/qt/component.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'component.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9AppStream9ComponentE_t {};
} // unnamed namespace

template <> constexpr inline auto AppStream::Component::qt_create_metaobjectdata<qt_meta_tag_ZN9AppStream9ComponentE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AppStream::Component",
        "Kind",
        "KindUnknown",
        "KindGeneric",
        "KindDesktopApp",
        "KindConsoleApp",
        "KindWebApp",
        "KindService",
        "KindAddon",
        "KindRuntime",
        "KindFont",
        "KindCodec",
        "KindInputMethod",
        "KindOperatingSystem",
        "KindFirmware",
        "KindDriver",
        "KindLocalization",
        "KindRepository",
        "KindIconTheme",
        "MergeKind",
        "MergeKindNone",
        "MergeKindReplace",
        "MergeKindAppend",
        "MergeKindRemoveComponent",
        "UrlKind",
        "UrlKindUnknown",
        "UrlKindHomepage",
        "UrlKindBugtracker",
        "UrlKindFaq",
        "UrlKindHelp",
        "UrlKindDonation",
        "UrlKindTranslate",
        "UrlKindContact",
        "UrlKindVcsBrowser",
        "UrlKindContribute",
        "Scope",
        "ScopeUnknown",
        "ScopeSystem",
        "ScopeUser"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Kind'
        QtMocHelpers::EnumData<enum Kind>(1, 1, QMC::EnumFlags{}).add({
            {    2, Kind::KindUnknown },
            {    3, Kind::KindGeneric },
            {    4, Kind::KindDesktopApp },
            {    5, Kind::KindConsoleApp },
            {    6, Kind::KindWebApp },
            {    7, Kind::KindService },
            {    8, Kind::KindAddon },
            {    9, Kind::KindRuntime },
            {   10, Kind::KindFont },
            {   11, Kind::KindCodec },
            {   12, Kind::KindInputMethod },
            {   13, Kind::KindOperatingSystem },
            {   14, Kind::KindFirmware },
            {   15, Kind::KindDriver },
            {   16, Kind::KindLocalization },
            {   17, Kind::KindRepository },
            {   18, Kind::KindIconTheme },
        }),
        // enum 'MergeKind'
        QtMocHelpers::EnumData<enum MergeKind>(19, 19, QMC::EnumFlags{}).add({
            {   20, MergeKind::MergeKindNone },
            {   21, MergeKind::MergeKindReplace },
            {   22, MergeKind::MergeKindAppend },
            {   23, MergeKind::MergeKindRemoveComponent },
        }),
        // enum 'UrlKind'
        QtMocHelpers::EnumData<enum UrlKind>(24, 24, QMC::EnumFlags{}).add({
            {   25, UrlKind::UrlKindUnknown },
            {   26, UrlKind::UrlKindHomepage },
            {   27, UrlKind::UrlKindBugtracker },
            {   28, UrlKind::UrlKindFaq },
            {   29, UrlKind::UrlKindHelp },
            {   30, UrlKind::UrlKindDonation },
            {   31, UrlKind::UrlKindTranslate },
            {   32, UrlKind::UrlKindContact },
            {   33, UrlKind::UrlKindVcsBrowser },
            {   34, UrlKind::UrlKindContribute },
        }),
        // enum 'Scope'
        QtMocHelpers::EnumData<enum Scope>(35, 35, QMC::EnumFlags{}).add({
            {   36, Scope::ScopeUnknown },
            {   37, Scope::ScopeSystem },
            {   38, Scope::ScopeUser },
        }),
    };
    return QtMocHelpers::metaObjectData<Component, qt_meta_tag_ZN9AppStream9ComponentE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AppStream::Component::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9AppStream9ComponentE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9AppStream9ComponentE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9AppStream9ComponentE_t>.metaTypes,
    nullptr
} };

QT_WARNING_POP
